@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -23,6 +23,7 @@
 #include "mutt.h"
 #include "mutt_curses.h"
 #include "mutt_menu.h"
+#include "attach.h"
 #include "buffy.h"
 #include "mapping.h"
 #include "sort.h"
@@ -560,8 +561,8 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
     {
       init_state (&state, NULL);
       state.imap_browse = 1;
-      imap_browse (f, &state);
-      strfcpy (LastDir, state.folder, sizeof (LastDir));
+      if (!imap_browse (f, &state))
+        strfcpy (LastDir, state.folder, sizeof (LastDir));
     }
     else
     {
@@ -612,6 +613,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
       init_state (&state, NULL);
       state.imap_browse = 1;
       imap_browse (LastDir, &state);
+      browser_sort (&state);
     }
 #endif
   }
@@ -756,6 +758,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	      init_state (&state, NULL);
 	      state.imap_browse = 1;
 	      imap_browse (LastDir, &state);
+	      browser_sort (&state);
 	      menu->data = state.entry;
 	    }
 	    else
@@ -867,6 +870,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	  init_state (&state, NULL);
 	  state.imap_browse = 1;
 	  imap_browse (LastDir, &state);
+	  browser_sort (&state);
 	  menu->data = state.entry;
 	  menu->current = 0; 
 	  menu->top = 0; 
@@ -887,6 +891,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	    init_state (&state, NULL);
 	    state.imap_browse = 1;
 	    imap_browse (LastDir, &state);
+	    browser_sort (&state);
 	    menu->data = state.entry;
 	    menu->current = 0;
 	    menu->top = 0;
@@ -958,6 +963,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	    init_state (&state, NULL);
 	    state.imap_browse = 1;
 	    imap_browse (LastDir, &state);
+	    browser_sort (&state);
 	    menu->data = state.entry;
 	    menu->current = 0; 
 	    menu->top = 0; 
@@ -1037,6 +1043,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	      init_state (&state, NULL);
 	      state.imap_browse = 1;
 	      imap_browse (LastDir, &state);
+	      browser_sort (&state);
 	      menu->data = state.entry;
 	      init_menu (&state, menu, title, sizeof (title), buffy);
 	    }
@@ -1122,6 +1129,7 @@ void _mutt_select_file (char *f, size_t flen, int flags, char ***files, int *num
 	  init_state (&state, NULL);
 	  state.imap_browse = 1;
 	  imap_browse (LastDir, &state);
+	  browser_sort (&state);
 	  menu->data = state.entry;
 	}
 #endif
