@@ -18,7 +18,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -504,7 +504,7 @@ int crypt_write_signed(BODY *a, STATE *s, const char *tempfile)
     return -1;
   }
       
-  fseek (s->fpin, a->hdr_offset, 0);
+  fseeko (s->fpin, a->hdr_offset, 0);
   bytes = a->length + a->offset - a->hdr_offset;
   hadcr = 0;
   while (bytes > 0)
@@ -552,7 +552,7 @@ void convert_to_7bit (BODY *a)
 	convert_to_7bit (a->parts);
     } 
     else if (a->type == TYPEMESSAGE &&
-	     mutt_strcasecmp(a->subtype, "delivery-status"))
+	     ascii_strcasecmp(a->subtype, "delivery-status"))
     {
       if(a->encoding != ENC7BIT)
 	mutt_message_to_7bit (a, NULL);
@@ -630,7 +630,7 @@ void crypt_extract_keys_from_messages (HEADER * h)
 	  fflush(fpout);
 
           if (Context->hdrs[Context->v2r[i]]->env->from)
-	    tmp = mutt_expand_aliases (h->env->from);
+	    tmp = mutt_expand_aliases (Context->hdrs[Context->v2r[i]]->env->from);
 	  else if (Context->hdrs[Context->v2r[i]]->env->sender)
 	    tmp = mutt_expand_aliases (Context->hdrs[Context->v2r[i]]
                                                     ->env->sender);
@@ -808,15 +808,15 @@ int mutt_signed_handler (BODY *a, STATE *s)
   
   if ((WithCrypto & APPLICATION_PGP)
       && protocol_major == TYPEAPPLICATION
-      && !mutt_strcasecmp (protocol_minor, "pgp-signature"))
+      && !ascii_strcasecmp (protocol_minor, "pgp-signature"))
     ;
   else if ((WithCrypto & APPLICATION_SMIME)
            && protocol_major == TYPEAPPLICATION
-	   && !(mutt_strcasecmp (protocol_minor, "x-pkcs7-signature")
-	       && mutt_strcasecmp (protocol_minor, "pkcs7-signature")))
+	   && !(ascii_strcasecmp (protocol_minor, "x-pkcs7-signature")
+	       && ascii_strcasecmp (protocol_minor, "pkcs7-signature")))
     ;
   else if (protocol_major == TYPEMULTIPART
-	   && !mutt_strcasecmp (protocol_minor, "mixed"))
+	   && !ascii_strcasecmp (protocol_minor, "mixed"))
     ;
   else
   {
@@ -840,7 +840,7 @@ int mutt_signed_handler (BODY *a, STATE *s)
 	{
 	  if ((WithCrypto & APPLICATION_PGP)
               && signatures[i]->type == TYPEAPPLICATION 
-	      && !mutt_strcasecmp (signatures[i]->subtype, "pgp-signature"))
+	      && !ascii_strcasecmp (signatures[i]->subtype, "pgp-signature"))
 	  {
 	    if (crypt_pgp_verify_one (signatures[i], s, tempfile) != 0)
 	      goodsig = 0;
@@ -850,8 +850,8 @@ int mutt_signed_handler (BODY *a, STATE *s)
 
 	  if ((WithCrypto & APPLICATION_SMIME)
               && signatures[i]->type == TYPEAPPLICATION 
-	      && (!mutt_strcasecmp(signatures[i]->subtype, "x-pkcs7-signature")
-		  || !mutt_strcasecmp(signatures[i]->subtype, "pkcs7-signature")))
+	      && (!ascii_strcasecmp(signatures[i]->subtype, "x-pkcs7-signature")
+		  || !ascii_strcasecmp(signatures[i]->subtype, "pkcs7-signature")))
 	  {
 	    if (crypt_smime_verify_one (signatures[i], s, tempfile) != 0)
 	      goodsig = 0;
