@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 WHERE void (*mutt_error) (const char *, ...);
@@ -67,10 +67,15 @@ WHERE char *Ispell;
 WHERE char *Locale;
 WHERE char *MailcapPath;
 WHERE char *Maildir;
+#if defined(USE_IMAP) || defined(USE_POP)
+WHERE char *MessageCachedir;
+#endif
 #if USE_HCACHE
 WHERE char *HeaderCache;
+#if HAVE_GDBM || HAVE_DB4
 WHERE char *HeaderCachePageSize;
-#endif
+#endif /* HAVE_GDBM || HAVE_DB4 */
+#endif /* USE_HCACHE */
 WHERE char *MhFlagged;
 WHERE char *MhReplied;
 WHERE char *MhUnseen;
@@ -112,17 +117,17 @@ WHERE char *Signature;
 WHERE char *SimpleSearch;
 WHERE char *Spoolfile;
 WHERE char *SpamSep;
-#if defined(USE_SSL) || defined(USE_GNUTLS)
+#if defined(USE_SSL)
 WHERE char *SslCertFile INITVAL (NULL);
 #endif
-#ifdef USE_SSL
+#ifdef USE_SSL_OPENSSL
 WHERE char *SslClientCert INITVAL (NULL);
 WHERE LIST *SslSessionCerts INITVAL (NULL);
 #endif
 #if defined(USE_SSL)
 WHERE char *SslEntropyFile INITVAL (NULL);
 #endif
-#ifdef USE_GNUTLS
+#ifdef USE_SSL_GNUTLS
 WHERE short SslDHPrimeBits;
 WHERE char *SslCACertFile INITVAL (NULL);
 #endif
@@ -136,8 +141,17 @@ WHERE char *Visual;
 WHERE char *CurrentFolder;
 WHERE char *LastFolder;
 
+
+WHERE const char *ReleaseDate;
+
+WHERE HASH *Groups;
+
 WHERE LIST *AutoViewList INITVAL(0);
 WHERE LIST *AlternativeOrderList INITVAL(0);
+WHERE LIST *AttachAllow INITVAL(0);
+WHERE LIST *AttachExclude INITVAL(0);
+WHERE LIST *InlineAllow INITVAL(0);
+WHERE LIST *InlineExclude INITVAL(0);
 WHERE LIST *HeaderOrderList INITVAL(0);
 WHERE LIST *Ignore INITVAL(0);
 WHERE LIST *MimeLookupList INITVAL(0);
@@ -152,6 +166,7 @@ WHERE RX_LIST *UnSubscribedLists INITVAL(0);
 WHERE SPAM_LIST *SpamList INITVAL(0);
 WHERE RX_LIST *NoSpamList INITVAL(0);
 
+
 /* bit vector for boolean variables */
 #ifdef MAIN_C
 unsigned char Options[(OPTMAX + 7)/8];
@@ -262,4 +277,5 @@ extern const char *Months[];
 #include "buffy.h"
 #include "sort.h"
 #include "mutt_crypt.h"
+#include "reldate.h"
 #endif /* MAIN_C */
