@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2002 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 /* Mutt browser support routines */
@@ -45,6 +45,7 @@ static int browse_verify_namespace (IMAP_DATA* idata,
 int imap_browse (char* path, struct browser_state* state)
 {
   IMAP_DATA* idata;
+  IMAP_LIST list;
   char buf[LONG_STRING];
   char buf2[LONG_STRING];
   char nsbuf[LONG_STRING];
@@ -57,10 +58,7 @@ int imap_browse (char* path, struct browser_state* state)
   int nsup;
   char ctmp;
   int nns = 0;
-  char *cur_folder;
   short showparents = 0;
-  int noselect;
-  int noinferiors;
   int save_lsub;
   IMAP_MBOX mx;
 
@@ -98,6 +96,7 @@ int imap_browse (char* path, struct browser_state* state)
   /* skip check for parents when at the root */
   if (mx.mbox && mx.mbox[0] != '\0')
   {
+    int rc;
     imap_fix_path (idata, mx.mbox, mbox, sizeof (mbox));
     imap_munge_mbox_name (buf, sizeof (buf), mbox);
     imap_unquote_string(buf); /* As kludgy as it gets */
@@ -113,29 +112,28 @@ int imap_browse (char* path, struct browser_state* state)
     {
       snprintf (buf, sizeof (buf), "%s \"\" \"%s\"", list_cmd, mbox);
       imap_cmd_start (idata, buf);
-      do 
+      idata->cmddata = &list;
+      do
       {
-        if (imap_parse_list_response (idata, &cur_folder, &noselect,
-            &noinferiors, &idata->delim) != 0)
-	  goto fail;
-
-        if (cur_folder)
+	list.name = 0;
+        rc = imap_cmd_step (idata);
+        if (rc == IMAP_CMD_CONTINUE && list.name)
         {
-          imap_unmunge_mbox_name (cur_folder);
-
-          if (!noinferiors && cur_folder[0] &&
-            (n = strlen (mbox)) < LONG_STRING-1)
+          if (!list.noinferiors && list.name[0] &&
+              !imap_mxcmp (list.name, mbox) &&
+            (n = strlen (mbox)) < sizeof (mbox) - 1)
           {
-            mbox[n++] = idata->delim;
+            mbox[n++] = list.delim;
             mbox[n] = '\0';
           }
         }
       }
-      while (ascii_strncmp (idata->cmd.buf, idata->cmd.seq, SEQLEN));
+      while (rc == IMAP_CMD_CONTINUE);
+      idata->cmddata = NULL;
     }
 
     /* if we're descending a folder, mark it as current in browser_state */
-    if (mbox[n-1] == idata->delim)
+    if (mbox[n-1] == list.delim)
     {
       /* don't show parents in the home namespace */
       if (!home_namespace)
@@ -151,7 +149,7 @@ int imap_browse (char* path, struct browser_state* state)
      *  and tack on delimiter ourselves.
      * Further note: UW-IMAP servers return nothing when asked for 
      *  NAMESPACES without delimiters at the end. Argh! */
-    for (n--; n >= 0 && mbox[n] != idata->delim ; n--);
+    for (n--; n >= 0 && mbox[n] != list.delim ; n--);
     if (n > 0)			/* "aaaa/bbbb/" -> "aaaa" */
     {
       /* forget the check, it is too delicate (see above). Have we ever
@@ -162,7 +160,7 @@ int imap_browse (char* path, struct browser_state* state)
       if (showparents)
       {
 	dprint (3, (debugfile, "imap_init_browse: adding parent %s\n", mbox));
-	imap_add_folder (idata->delim, mbox, 1, 0, state, 1);
+	imap_add_folder (list.delim, mbox, 1, 0, state, 1);
       }
 
       /* if our target isn't a folder, we are in our superior */
@@ -342,7 +340,7 @@ int imap_mailbox_rename(const char* mailbox)
   }
 
   if (imap_rename_mailbox (idata, &mx, newname) < 0) {
-    mutt_error (_("Rename failed: %s"), imap_get_qualifier (idata->cmd.buf));
+    mutt_error (_("Rename failed: %s"), imap_get_qualifier (idata->buf));
     mutt_sleep (1);
     goto fail;
   }
@@ -361,10 +359,9 @@ int imap_mailbox_rename(const char* mailbox)
 static int browse_add_list_result (IMAP_DATA* idata, const char* cmd,
   struct browser_state* state, short isparent)
 {
-  char *name;
-  int noselect;
-  int noinferiors;
+  IMAP_LIST list;
   IMAP_MBOX mx;
+  int rc;
 
   if (imap_parse_path (state->folder, &mx))
   {
@@ -374,31 +371,27 @@ static int browse_add_list_result (IMAP_DATA* idata, const char* cmd,
   }
 
   imap_cmd_start (idata, cmd);
-
-  do 
+  idata->cmddata = &list;
+  do
   {
-    if (imap_parse_list_response(idata, &name, &noselect, &noinferiors,
-        &idata->delim) != 0)
-    {
-      FREE (&mx.mbox);
-      return -1;
-    }
+    rc = imap_cmd_step (idata);
 
-    if (name)
+    if (rc == IMAP_CMD_CONTINUE && list.name)
     {
       /* Let a parent folder never be selectable for navigation */
       if (isparent)
-        noselect = 1;
+        list.noselect = 1;
       /* prune current folder from output */
-      if (isparent || mutt_strncmp (name, mx.mbox, strlen (name)))
-        imap_add_folder (idata->delim, name, noselect, noinferiors, state,
-          isparent);
+      if (isparent || mutt_strncmp (list.name, mx.mbox, strlen (list.name)))
+        imap_add_folder (list.delim, list.name, list.noselect, list.noinferiors,
+                         state, isparent);
     }
   }
-  while ((ascii_strncmp (idata->cmd.buf, idata->cmd.seq, SEQLEN) != 0));
+  while (rc == IMAP_CMD_CONTINUE);
+  idata->cmddata = NULL;
 
   FREE (&mx.mbox);
-  return 0;
+  return rc == IMAP_CMD_OK ? 0 : -1;
 }
 
 /* imap_add_folder: add a folder name to the browser list, formatting it as
@@ -491,7 +484,7 @@ static int browse_get_namespace (IMAP_DATA* idata, char* nsbuf, int nsblen,
     if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
       break;
 
-    s = imap_next_word (idata->cmd.buf);
+    s = imap_next_word (idata->buf);
     if (ascii_strncasecmp ("NAMESPACE", s, 9) == 0)
     {
       /* There are three sections to the response, User, Other, Shared,
@@ -512,7 +505,7 @@ static int browse_get_namespace (IMAP_DATA* idata, char* nsbuf, int nsblen,
 	    if (*s == '\"')
 	    {
 	      s++;
-	      while (*s && *s != '\"') 
+	      while (*s && *s != '\"' && n < sizeof (ns) - 1) 
 	      {
 		if (*s == '\\')
 		  s++;
@@ -523,12 +516,14 @@ static int browse_get_namespace (IMAP_DATA* idata, char* nsbuf, int nsblen,
 		s++;
 	    }
 	    else
-	      while (*s && !ISSPACE (*s)) 
+	      while (*s && !ISSPACE (*s) && n < sizeof (ns) - 1)
 	      {
 		ns[n++] = *s;
 		s++;
 	      }
 	    ns[n] = '\0';
+	    if (n == sizeof (ns) - 1)
+	      dprint (1, (debugfile, "browse_get_namespace: too long: [%s]\n", ns));
 	    /* delim? */
 	    s = imap_next_word (s);
 	    /* delimiter is meaningless if namespace is "". Why does
@@ -579,9 +574,9 @@ static int browse_verify_namespace (IMAP_DATA* idata,
   IMAP_NAMESPACE_INFO *nsi, int nns)
 {
   char buf[LONG_STRING];
+  IMAP_LIST list;
   int i = 0;
-  char *name;
-  char delim;
+  int rc;
 
   for (i = 0; i < nns; i++, nsi++)
   {
@@ -597,17 +592,20 @@ static int browse_verify_namespace (IMAP_DATA* idata,
 		option (OPTIMAPLSUB) ? "LSUB" : "LIST", nsi->prefix);
 
     imap_cmd_start (idata, buf);
-
+    idata->cmddata = &list;
     nsi->listable = 0;
     nsi->home_namespace = 0;
     do 
     {
-      if (imap_parse_list_response(idata, &name, &nsi->noselect,
-          &nsi->noinferiors, &delim) != 0)
-	return -1;
-      nsi->listable |= (name != NULL);
+      rc = imap_cmd_step (idata);
+      if (rc == IMAP_CMD_CONTINUE && list.name)
+        nsi->listable |= (list.name[0] != '\0');
     }
-    while ((ascii_strncmp (idata->cmd.buf, idata->cmd.seq, SEQLEN) != 0));
+    while (rc == IMAP_CMD_CONTINUE);
+    idata->cmddata = NULL;
+
+    if (rc != IMAP_CMD_OK)
+      return -1;
   }
 
   return 0;
