@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1996-8 Michael R. Elkins <me@mutt.org>
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -15,7 +15,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 /* command.c: routines for sending commands to an IMAP server and parsing
@@ -29,6 +29,7 @@
 #include "imap_private.h"
 #include "message.h"
 #include "mx.h"
+#include "buffy.h"
 
 #include <ctype.h>
 #include <stdlib.h>
@@ -36,15 +37,18 @@
 #define IMAP_CMD_BUFSIZE 512
 
 /* forward declarations */
+static IMAP_COMMAND* cmd_new (IMAP_DATA* idata);
+static int cmd_status (const char *s);
 static void cmd_handle_fatal (IMAP_DATA* idata);
 static int cmd_handle_untagged (IMAP_DATA* idata);
-static void cmd_make_sequence (IMAP_DATA* idata);
-static void cmd_parse_capabilities (IMAP_DATA* idata, char* s);
+static void cmd_parse_capability (IMAP_DATA* idata, char* s);
 static void cmd_parse_expunge (IMAP_DATA* idata, const char* s);
+static void cmd_parse_list (IMAP_DATA* idata, char* s);
 static void cmd_parse_lsub (IMAP_DATA* idata, char* s);
 static void cmd_parse_fetch (IMAP_DATA* idata, char* s);
-static void cmd_parse_myrights (IMAP_DATA* idata, char* s);
-static void cmd_parse_search (IMAP_DATA* idata, char* s);
+static void cmd_parse_myrights (IMAP_DATA* idata, const char* s);
+static void cmd_parse_search (IMAP_DATA* idata, const char* s);
+static void cmd_parse_status (IMAP_DATA* idata, char* s);
 
 static char *Capabilities[] = {
   "IMAP4",
@@ -57,18 +61,16 @@ static char *Capabilities[] = {
   "AUTH=ANONYMOUS",
   "STARTTLS",
   "LOGINDISABLED",
+  "IDLE",
+  "SASL-IR",
 
   NULL
 };
 
-/* imap_cmd_start: Given an IMAP command, send it to the server.
- *   Currently a minor convenience, but helps to route all IMAP commands
- *   through a single interface. */
-int imap_cmd_start (IMAP_DATA* idata, const char* cmd)
+/* imap_cmd_queue: Add command to command queue. Fails if the queue is full. */
+int imap_cmd_queue (IMAP_DATA* idata, const char* cmdstr)
 {
-  char* out;
-  int outlen;
-  int rc;
+  IMAP_COMMAND* cmd;
 
   if (idata->status == IMAP_FATAL)
   {
@@ -76,15 +78,34 @@ int imap_cmd_start (IMAP_DATA* idata, const char* cmd)
     return IMAP_CMD_BAD;
   }
 
-  cmd_make_sequence (idata);
-  /* seq, space, cmd, \r\n\0 */
-  outlen = strlen (idata->cmd.seq) + strlen (cmd) + 4;
-  out = (char*) safe_malloc (outlen);
-  snprintf (out, outlen, "%s %s\r\n", idata->cmd.seq, cmd);
+  if (!(cmd = cmd_new (idata)))
+    return IMAP_CMD_BAD;
 
-  rc = mutt_socket_write (idata->conn, out);
+  if (mutt_buffer_printf (idata->cmdbuf, "%s%s %s\r\n",
+      idata->state == IMAP_IDLE ? "DONE\r\n" : "", cmd->seq, cmdstr) < 0)
+    return IMAP_CMD_BAD;
 
-  FREE (&out);
+  if (idata->state == IMAP_IDLE)
+    idata->state = IMAP_SELECTED;
+
+  return 0;
+}
+
+/* imap_cmd_start: Given an IMAP command, send it to the server.
+ *   If cmdstr is NULL, sends queued commands. */
+int imap_cmd_start (IMAP_DATA* idata, const char* cmdstr)
+{
+  int rc;
+
+  if (cmdstr && (rc = imap_cmd_queue (idata, cmdstr)) < 0)
+    return rc;
+
+  /* don't write old or empty commands */
+  if (idata->cmdbuf->dptr == idata->cmdbuf->data)
+    return IMAP_CMD_BAD;
+
+  rc = mutt_socket_write (idata->conn, idata->cmdbuf->data);
+  idata->cmdbuf->dptr = idata->cmdbuf->data;
 
   return (rc < 0) ? IMAP_CMD_BAD : 0;
 }
@@ -95,9 +116,9 @@ int imap_cmd_start (IMAP_DATA* idata, const char* cmd)
  *   large!). */
 int imap_cmd_step (IMAP_DATA* idata)
 {
-  IMAP_COMMAND* cmd = &idata->cmd;
-  unsigned int len = 0;
+  size_t len = 0;
   int c;
+  int rc;
 
   if (idata->status == IMAP_FATAL)
   {
@@ -109,18 +130,18 @@ int imap_cmd_step (IMAP_DATA* idata)
    * line */
   do
   {
-    if (len == cmd->blen)
+    if (len == idata->blen)
     {
-      safe_realloc (&cmd->buf, cmd->blen + IMAP_CMD_BUFSIZE);
-      cmd->blen = cmd->blen + IMAP_CMD_BUFSIZE;
+      safe_realloc (&idata->buf, idata->blen + IMAP_CMD_BUFSIZE);
+      idata->blen = idata->blen + IMAP_CMD_BUFSIZE;
       dprint (3, (debugfile, "imap_cmd_step: grew buffer to %u bytes\n",
-		  cmd->blen));
+		  idata->blen));
     }
 
     /* back up over '\0' */
     if (len)
       len--;
-    c = mutt_socket_readln (cmd->buf + len, cmd->blen - len, idata->conn);
+    c = mutt_socket_readln (idata->buf + len, idata->blen - len, idata->conn);
     if (c <= 0)
     {
       dprint (1, (debugfile, "imap_cmd_step: Error reading server response.\n"));
@@ -133,45 +154,48 @@ int imap_cmd_step (IMAP_DATA* idata)
   /* if we've read all the way to the end of the buffer, we haven't read a
    * full line (mutt_socket_readln strips the \r, so we always have at least
    * one character free when we've read a full line) */
-  while (len == cmd->blen);
+  while (len == idata->blen);
 
   /* don't let one large string make cmd->buf hog memory forever */
-  if ((cmd->blen > IMAP_CMD_BUFSIZE) && (len <= IMAP_CMD_BUFSIZE))
+  if ((idata->blen > IMAP_CMD_BUFSIZE) && (len <= IMAP_CMD_BUFSIZE))
   {
-    safe_realloc (&cmd->buf, IMAP_CMD_BUFSIZE);
-    cmd->blen = IMAP_CMD_BUFSIZE;
-    dprint (3, (debugfile, "imap_cmd_step: shrank buffer to %u bytes\n", cmd->blen));
+    safe_realloc (&idata->buf, IMAP_CMD_BUFSIZE);
+    idata->blen = IMAP_CMD_BUFSIZE;
+    dprint (3, (debugfile, "imap_cmd_step: shrank buffer to %u bytes\n", idata->blen));
   }
 
-  idata->lastread = time(NULL);
+  idata->lastread = time (NULL);
 
   /* handle untagged messages. The caller still gets its shot afterwards. */
-  if (!ascii_strncmp (cmd->buf, "* ", 2) &&
+  if (!ascii_strncmp (idata->buf, "* ", 2) &&
       cmd_handle_untagged (idata))
     return IMAP_CMD_BAD;
 
   /* server demands a continuation response from us */
-  if (!ascii_strncmp (cmd->buf, "+ ", 2))
-  {
+  if (idata->buf[0] == '+')
     return IMAP_CMD_RESPOND;
-  }
 
-  /* tagged completion code */
-  if (!ascii_strncmp (cmd->buf, cmd->seq, SEQLEN))
+  /* tagged completion code. TODO: I believe commands will always be completed
+   * in order, but will have to double-check when I have net again */
+  rc = IMAP_CMD_CONTINUE;
+  if (!ascii_strncmp (idata->buf, idata->cmds[idata->lastcmd].seq, SEQLEN))
   {
-    imap_cmd_finish (idata);
-    return imap_code (cmd->buf) ? IMAP_CMD_OK : IMAP_CMD_NO;
+    idata->cmds[idata->lastcmd].state = cmd_status (idata->buf);
+    idata->lastcmd = (idata->lastcmd + 1) % IMAP_PIPELINE_DEPTH;
+    if (idata->lastcmd == idata->nextcmd)
+    {
+      rc = cmd_status (idata->buf);
+      imap_cmd_finish (idata);
+    }
   }
 
-  return IMAP_CMD_CONTINUE;
+  return rc;
 }
 
 /* imap_code: returns 1 if the command result was OK, or 0 if NO or BAD */
 int imap_code (const char *s)
 {
-  s += SEQLEN;
-  SKIPWS (s);
-  return (ascii_strncasecmp ("OK", s, 2) == 0);
+  return cmd_status (s) == IMAP_CMD_OK;
 }
 
 /* imap_exec: execute a command, and wait for the response from the server.
@@ -182,10 +206,8 @@ int imap_code (const char *s)
  *   IMAP_CMD_PASS: command contains a password. Suppress logging.
  * Return 0 on success, -1 on Failure, -2 on OK Failure
  */
-int imap_exec (IMAP_DATA* idata, const char* cmd, int flags)
+int imap_exec (IMAP_DATA* idata, const char* cmdstr, int flags)
 {
-  char* out;
-  int outlen;
   int rc;
 
   if (idata->status == IMAP_FATAL)
@@ -194,16 +216,16 @@ int imap_exec (IMAP_DATA* idata, const char* cmd, int flags)
     return -1;
   }
 
-  /* create sequence for command */
-  cmd_make_sequence (idata);
-  /* seq, space, cmd, \r\n\0 */
-  outlen = strlen (idata->cmd.seq) + strlen (cmd) + 4;
-  out = (char*) safe_malloc (outlen);
-  snprintf (out, outlen, "%s %s\r\n", idata->cmd.seq, cmd);
-
-  rc = mutt_socket_write_d (idata->conn, out,
+  if (cmdstr && (rc = imap_cmd_queue (idata, cmdstr)) < 0)
+    return rc;
+  
+  /* don't write old or empty commands */
+  if (idata->cmdbuf->dptr == idata->cmdbuf->data)
+    return IMAP_CMD_BAD;
+  
+  rc = mutt_socket_write_d (idata->conn, idata->cmdbuf->data, -1,
     flags & IMAP_CMD_PASS ? IMAP_LOG_PASS : IMAP_LOG_CMD);
-  FREE (&out);
+  idata->cmdbuf->dptr = idata->cmdbuf->data;
 
   if (rc < 0)
   {
@@ -223,23 +245,13 @@ int imap_exec (IMAP_DATA* idata, const char* cmd, int flags)
     if (flags & IMAP_CMD_FAIL_OK)
       return -2;
 
-    dprint (1, (debugfile, "imap_exec: command failed: %s\n", idata->cmd.buf));
+    dprint (1, (debugfile, "imap_exec: command failed: %s\n", idata->buf));
     return -1;
   }
 
   return 0;
 }
 
-/* imap_cmd_running: Returns whether an IMAP command is in progress. */
-int imap_cmd_running (IMAP_DATA* idata)
-{
-  if (idata->cmd.state == IMAP_CMD_CONTINUE ||
-      idata->cmd.state == IMAP_CMD_RESPOND)
-    return 1;
-
-  return 0;
-}
-
 /* imap_cmd_finish: Attempts to perform cleanup (eg fetch new mail if
  *   detected, do expunge). Called automatically by imap_cmd_step, but
  *   may be called at any time. Called by imap_check_mailbox just before
@@ -252,7 +264,7 @@ void imap_cmd_finish (IMAP_DATA* idata)
     return;
   }
 
-  if (!(idata->state == IMAP_SELECTED) || idata->ctx->closing)
+  if (!(idata->state >= IMAP_SELECTED) || idata->ctx->closing)
     return;
   
   if (idata->reopen & IMAP_REOPEN_ALLOW)
@@ -286,12 +298,49 @@ void imap_cmd_finish (IMAP_DATA* idata)
   idata->status = 0;
 }
 
+/* sets up a new command control block and adds it to the queue.
+ * Returns NULL if the pipeline is full. */
+static IMAP_COMMAND* cmd_new (IMAP_DATA* idata)
+{
+  IMAP_COMMAND* cmd;
+
+  if ((idata->nextcmd + 1) % IMAP_PIPELINE_DEPTH == idata->lastcmd)
+  {
+    dprint (2, (debugfile, "cmd_new: IMAP command queue full\n"));
+    return NULL;
+  }
+
+  cmd = idata->cmds + idata->nextcmd;
+  idata->nextcmd = (idata->nextcmd + 1) % IMAP_PIPELINE_DEPTH;
+
+  snprintf (cmd->seq, sizeof (cmd->seq), "a%04u", idata->seqno++);
+  if (idata->seqno > 9999)
+    idata->seqno = 0;
+
+  cmd->state = IMAP_CMD_NEW;
+
+  return cmd;
+}
+
+/* parse response line for tagged OK/NO/BAD */
+static int cmd_status (const char *s)
+{
+  s = imap_next_word((char*)s);
+  
+  if (!ascii_strncasecmp("OK", s, 2))
+    return IMAP_CMD_OK;
+  if (!ascii_strncasecmp("NO", s, 2))
+    return IMAP_CMD_NO;
+
+  return IMAP_CMD_BAD;
+}
+
 /* cmd_handle_fatal: when IMAP_DATA is in fatal state, do what we can */
 static void cmd_handle_fatal (IMAP_DATA* idata)
 {
   idata->status = IMAP_FATAL;
 
-  if ((idata->state == IMAP_SELECTED) &&
+  if ((idata->state >= IMAP_SELECTED) &&
       (idata->reopen & IMAP_REOPEN_ALLOW))
   {
     mx_fastclose_mailbox (idata->ctx);
@@ -300,7 +349,7 @@ static void cmd_handle_fatal (IMAP_DATA* idata)
     idata->state = IMAP_DISCONNECTED;
   }
 
-  if (idata->state != IMAP_SELECTED)
+  if (idata->state < IMAP_SELECTED)
   {
     idata->state = IMAP_DISCONNECTED;
     mutt_socket_close (idata->conn);
@@ -315,9 +364,9 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
   char* pn;
   int count;
 
-  s = imap_next_word (idata->cmd.buf);
+  s = imap_next_word (idata->buf);
 
-  if ((idata->state == IMAP_SELECTED) && isdigit ((unsigned char) *s))
+  if ((idata->state >= IMAP_SELECTED) && isdigit ((unsigned char) *s))
   {
     pn = s;
     s = imap_next_word (s);
@@ -366,13 +415,19 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
       cmd_parse_fetch (idata, pn);
   }
   else if (ascii_strncasecmp ("CAPABILITY", s, 10) == 0)
-    cmd_parse_capabilities (idata, s);
+    cmd_parse_capability (idata, s);
+  else if (!ascii_strncasecmp ("OK [CAPABILITY", s, 14))
+    cmd_parse_capability (idata, imap_next_word (s));
+  else if (ascii_strncasecmp ("LIST", s, 4) == 0)
+    cmd_parse_list (idata, s);
   else if (ascii_strncasecmp ("LSUB", s, 4) == 0)
     cmd_parse_lsub (idata, s);
   else if (ascii_strncasecmp ("MYRIGHTS", s, 8) == 0)
     cmd_parse_myrights (idata, s);
   else if (ascii_strncasecmp ("SEARCH", s, 6) == 0)
     cmd_parse_search (idata, s);
+  else if (ascii_strncasecmp ("STATUS", s, 6) == 0)
+    cmd_parse_status (idata, s);
   else if (ascii_strncasecmp ("BYE", s, 3) == 0)
   {
     dprint (2, (debugfile, "Handling BYE\n"));
@@ -402,24 +457,18 @@ static int cmd_handle_untagged (IMAP_DATA* idata)
   return 0;
 }
 
-/* cmd_make_sequence: make a tag suitable for starting an IMAP command */
-static void cmd_make_sequence (IMAP_DATA* idata)
-{
-  snprintf (idata->cmd.seq, sizeof (idata->cmd.seq), "a%04u", idata->seqno++);
-
-  if (idata->seqno > 9999)
-    idata->seqno = 0;
-}
-
 /* cmd_parse_capabilities: set capability bits according to CAPABILITY
  *   response */
-static void cmd_parse_capabilities (IMAP_DATA* idata, char* s)
+static void cmd_parse_capability (IMAP_DATA* idata, char* s)
 {
   int x;
+  char* bracket;
 
   dprint (2, (debugfile, "Handling CAPABILITY\n"));
 
   s = imap_next_word (s);
+  if ((bracket = strchr (s, ']')))
+    *bracket = '\0';
   FREE(&idata->capstr);
   idata->capstr = safe_strdup (s);
 
@@ -525,72 +574,111 @@ static void cmd_parse_fetch (IMAP_DATA* idata, char* s)
   }
 }
 
+static void cmd_parse_list (IMAP_DATA* idata, char* s)
+{
+  IMAP_LIST* list;
+  IMAP_LIST lb;
+  char delimbuf[5]; /* worst case: "\\"\0 */
+
+  if (idata->cmddata)
+    list = (IMAP_LIST*)idata->cmddata;
+  else
+    list = &lb;
+
+  memset (list, 0, sizeof (IMAP_LIST));
+
+  /* flags */
+  s = imap_next_word (s);
+  if (*s != '(')
+  {
+    dprint (1, (debugfile, "Bad LIST response\n"));
+    return;
+  }
+  s++;
+  while (*s)
+  {
+    if (!ascii_strncasecmp (s, "\\NoSelect", 9))
+      list->noselect = 1;
+    else if (!ascii_strncasecmp (s, "\\NoInferiors", 12))
+      list->noinferiors = 1;
+    /* See draft-gahrns-imap-child-mailbox-?? */
+    else if (!ascii_strncasecmp (s, "\\HasNoChildren", 14))
+      list->noinferiors = 1;
+    
+    s = imap_next_word (s);
+    if (*(s - 2) == ')')
+      break;
+  }
+
+  /* Delimiter */
+  if (ascii_strncasecmp (s, "NIL", 3))
+  {
+    delimbuf[0] = '\0';
+    safe_strcat (delimbuf, 5, s); 
+    imap_unquote_string (delimbuf);
+    list->delim = delimbuf[0];
+  }
+
+  /* Name */
+  s = imap_next_word (s);
+  imap_unmunge_mbox_name (s);
+  list->name = s;
+  
+  if (list->name[0] == '\0')
+  {
+    idata->delim = list->delim;
+    dprint (2, (debugfile, "Root delimiter: %c\n", idata->delim));
+  }
+}
+
 static void cmd_parse_lsub (IMAP_DATA* idata, char* s)
 {
   char buf[STRING];
   char errstr[STRING];
   BUFFER err, token;
   ciss_url_t url;
-  char *ep;
+  IMAP_LIST list;
+
+  if (idata->cmddata)
+  {
+    /* caller will handle response itself */
+    cmd_parse_list (idata, s);
+    return;
+  }
 
   if (!option (OPTIMAPCHECKSUBSCRIBED))
     return;
 
-  s = imap_next_word (s); /* flags */
-  
-  if (*s != '(')
-  {
-    dprint (1, (debugfile, "Bad LSUB response\n"));
+  idata->cmddata = &list;
+  cmd_parse_list (idata, s);
+  idata->cmddata = NULL;
+  if (!list.name)
     return;
-  }
 
-  s++;
-  ep = s;
-  for (ep = s; *ep && *ep != ')'; ep++)
-    ;
-  do
-  {
-    if (!ascii_strncasecmp (s, "\\NoSelect", 9))
-      return;
-    while (s < ep && *s != ' ' && *s != ')')
-      s++;
-    if (*s == ' ')
-      s++;
-  } while (s != ep);
+  dprint (2, (debugfile, "Subscribing to %s\n", list.name));
 
-  s = imap_next_word (s); /* delim */
-  s = imap_next_word (s); /* name */
-  
-  if (s)
-  {
-    imap_unmunge_mbox_name (s);
-    dprint (2, (debugfile, "Subscribing to %s\n", s));
-    
-    strfcpy (buf, "mailboxes \"", sizeof (buf));
-    mutt_account_tourl (&idata->conn->account, &url);
-    url.path = s;
-    if (!mutt_strcmp (url.user, ImapUser))
-      url.user = NULL;
-    url_ciss_tostring (&url, buf + 11, sizeof (buf) - 10, 0);
-    safe_strcat (buf, sizeof (buf), "\"");
-    memset (&token, 0, sizeof (token));
-    err.data = errstr;
-    err.dsize = sizeof (errstr);
-    if (mutt_parse_rc_line (buf, &token, &err))
-      dprint (1, (debugfile, "Error adding subscribed mailbox: %s\n", errstr));
-    FREE (&token.data);
-  }
-  else
-    dprint (1, (debugfile, "Bad LSUB response\n"));
+  strfcpy (buf, "mailboxes \"", sizeof (buf));
+  mutt_account_tourl (&idata->conn->account, &url);
+  url.path = list.name;
+  if (!mutt_strcmp (url.user, ImapUser))
+    url.user = NULL;
+  url_ciss_tostring (&url, buf + 11, sizeof (buf) - 10, 0);
+  safe_strcat (buf, sizeof (buf), "\"");
+  memset (&token, 0, sizeof (token));
+  err.data = errstr;
+  err.dsize = sizeof (errstr);
+  if (mutt_parse_rc_line (buf, &token, &err))
+    dprint (1, (debugfile, "Error adding subscribed mailbox: %s\n", errstr));
+  FREE (&token.data);
 }
 
 /* cmd_parse_myrights: set rights bits according to MYRIGHTS response */
-static void cmd_parse_myrights (IMAP_DATA* idata, char* s)
+static void cmd_parse_myrights (IMAP_DATA* idata, const char* s)
 {
   dprint (2, (debugfile, "Handling MYRIGHTS\n"));
 
-  s = imap_next_word (s);
-  s = imap_next_word (s);
+  s = imap_next_word ((char*)s);
+  s = imap_next_word ((char*)s);
 
   /* zero out current rights set */
   memset (idata->rights, 0, sizeof (idata->rights));
@@ -647,14 +735,14 @@ static int uid2msgno (IMAP_DATA* idata, unsigned int uid)
 }
 
 /* cmd_parse_search: store SEARCH response for later use */
-static void cmd_parse_search (IMAP_DATA* idata, char* s)
+static void cmd_parse_search (IMAP_DATA* idata, const char* s)
 {
   unsigned int uid;
   int msgno;
 
   dprint (2, (debugfile, "Handling SEARCH\n"));
 
-  while ((s = imap_next_word (s)) && *s != '\0')
+  while ((s = imap_next_word ((char*)s)) && *s != '\0')
   {
     uid = atoi (s);
     msgno = uid2msgno (idata, uid);
@@ -663,3 +751,123 @@ static void cmd_parse_search (IMAP_DATA* idata, char* s)
       idata->ctx->hdrs[uid2msgno (idata, uid)]->matched = 1;
   }
 }
+
+/* first cut: just do buffy update. Later we may wish to cache all
+ * mailbox information, even that not desired by buffy */
+static void cmd_parse_status (IMAP_DATA* idata, char* s)
+{
+  char* mailbox;
+  char* value;
+  BUFFY* inc;
+  IMAP_MBOX mx;
+  int count;
+  IMAP_STATUS *status, sb;
+  int olduv, oldun;
+
+  mailbox = imap_next_word (s);
+  s = imap_next_word (mailbox);
+  *(s - 1) = '\0';
+  imap_unmunge_mbox_name (mailbox);
+
+  if (!(status = imap_mboxcache_get (idata, mailbox)))
+  {
+    /* ugly interface - why should I look up what I just added? */
+    memset (&sb, 0, sizeof (IMAP_STATUS));
+    sb.name = mailbox;
+    idata->mboxcache = mutt_add_list_n (idata->mboxcache, &sb, sizeof (IMAP_STATUS));
+    status = imap_mboxcache_get (idata, mailbox);
+    status->name = safe_strdup (mailbox);
+  }
+  olduv = status->uidvalidity;
+  oldun = status->uidnext;
+
+  if (*s++ != '(')
+  {
+    dprint (1, (debugfile, "Error parsing STATUS\n"));
+    return;
+  }
+  while (*s && *s != ')')
+  {
+    value = imap_next_word (s);
+    count = strtol (value, &value, 10);
+
+    if (!ascii_strncmp ("MESSAGES", s, 8))
+      status->messages = count;
+    else if (!ascii_strncmp ("RECENT", s, 6))
+      status->recent = count;
+    else if (!ascii_strncmp ("UIDNEXT", s, 7))
+      status->uidnext = count;
+    else if (!ascii_strncmp ("UIDVALIDITY", s, 11))
+      status->uidvalidity = count;
+    else if (!ascii_strncmp ("UNSEEN", s, 6))
+      status->unseen = count;
+
+    s = value;
+    if (*s && *s != ')')
+      s = imap_next_word (s);
+  }
+  dprint (2, (debugfile, "%s (UIDVALIDITY: %d, UIDNEXT: %d) %d messages, %d recent, %d unseen\n",
+              status->name, status->uidvalidity, status->uidnext,
+              status->messages, status->recent, status->unseen));
+
+  /* caller is prepared to handle the result herself */
+  if (idata->cmddata)
+  {
+    memcpy (idata->cmddata, status, sizeof (IMAP_STATUS));
+    return;
+  }
+
+  dprint (2, (debugfile, "Running default STATUS handler\n"));
+
+  /* should perhaps move this code back to imap_buffy_check */
+  for (inc = Incoming; inc; inc = inc->next)
+  {
+    if (inc->magic != M_IMAP)
+      continue;
+    
+    if (imap_parse_path (inc->path, &mx) < 0)
+    {
+      dprint (1, (debugfile, "Error parsing mailbox %s, skipping\n", inc->path));
+      continue;
+    }
+    /* dprint (2, (debugfile, "Buffy entry: [%s] mbox: [%s]\n", inc->path, NONULL(mx.mbox))); */
+    
+    if (mutt_account_match (&idata->conn->account, &mx.account))
+    {
+      if (mx.mbox)
+      {
+	value = safe_strdup (mx.mbox);
+	imap_fix_path (idata, mx.mbox, value, mutt_strlen (value) + 1);
+	FREE (&mx.mbox);
+      }
+      else
+	value = safe_strdup ("INBOX");
+
+      if (value && !imap_mxcmp (mailbox, value))
+      {
+        dprint (2, (debugfile, "Found %s in buffy list (OV: %d ON: %d U: %d)\n",
+                    mailbox, olduv, oldun, status->unseen));
+        
+        if (olduv && olduv == status->uidvalidity)
+        {
+          if (oldun < status->uidnext)
+          {
+            inc->new = status->unseen;
+          }
+        }
+        else
+          inc->new = status->unseen;
+
+        /* forced back to keep detecting new mail until the mailbox is opened */
+        status->uidnext = oldun;
+
+        FREE (&value);
+        return;
+      }
+
+      FREE (&value);
+    }
+
+    FREE (&mx.mbox);
+  }
+}
