@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2001 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #ifndef _IMAP_PRIVATE_H
@@ -23,6 +23,7 @@
 #include "imap.h"
 #include "mutt_curses.h"
 #include "mutt_socket.h"
+#include "bcache.h"
 
 /* -- symbols -- */
 #define IMAP_PORT 143
@@ -44,10 +45,16 @@
 #define IMAP_CMD_CONTINUE (1)
 /* + */
 #define IMAP_CMD_RESPOND  (2)
+/* IMAP_COMMAND.state additions */
+#define IMAP_CMD_NEW    (3)
 
 /* number of entries in the hash table */
 #define IMAP_CACHE_LEN 10
 
+/* number of commands that can be batched into a single request
+ * ( - 1, for the easy way to detect ring buffer wrap) */
+#define IMAP_PIPELINE_DEPTH 15
+
 #define SEQLEN 5
 
 #define IMAP_REOPEN_ALLOW     (1<<0)
@@ -73,7 +80,10 @@ enum
   IMAP_DISCONNECTED = 0,
   IMAP_CONNECTED,
   IMAP_AUTHENTICATED,
-  IMAP_SELECTED
+  IMAP_SELECTED,
+  
+  /* and pseudo-states */
+  IMAP_IDLE
 };
 
 enum
@@ -113,6 +123,8 @@ enum
   AUTH_ANON,			/* AUTH=ANONYMOUS */
   STARTTLS,			/* RFC 2595: STARTTLS */
   LOGINDISABLED,		/*           LOGINDISABLED */
+  IDLE,                         /* RFC 2177: IDLE */
+  SASL_IR,                      /* SASL initial response draft */
 
   CAPMAX
 };
@@ -140,12 +152,31 @@ typedef struct
   int noinferiors;
 } IMAP_NAMESPACE_INFO;
 
+typedef struct
+{
+  char* name;
+
+  unsigned int messages;
+  unsigned int recent;
+  unsigned int uidnext;
+  unsigned int uidvalidity;
+  unsigned int unseen;
+} IMAP_STATUS;
+
+typedef struct
+{
+  char* name;
+  
+  char delim;
+  /* if we end up storing a lot of these we could turn this into a bitfield */
+  unsigned char noselect;
+  unsigned char noinferiors;
+} IMAP_LIST;
+
 /* IMAP command structure */
 typedef struct
 {
   char seq[SEQLEN+1];
-  char* buf;
-  unsigned int blen;
   int state;
 } IMAP_COMMAND;
 
@@ -167,8 +198,19 @@ typedef struct
   unsigned char capabilities[(CAPMAX + 7)/8];
   unsigned int seqno;
   time_t lastread; /* last time we read a command for the server */
-  /* who knows, one day we may run multiple commands in parallel */
-  IMAP_COMMAND cmd;
+  char* buf;
+  unsigned int blen;
+  void* cmddata; /* if set, the response parser will store results for
+                  * complicated commands here. */
+
+  /* command queue */
+  IMAP_COMMAND cmds[IMAP_PIPELINE_DEPTH];
+  int nextcmd;
+  int lastcmd;
+  BUFFER* cmdbuf;
+
+  /* cache IMAP_STATUS of visited mailboxes */
+  LIST* mboxcache;
 
   /* The following data is all specific to the currently SELECTED mbox */
   char delim;
@@ -179,10 +221,10 @@ typedef struct
   unsigned char rights[(RIGHTSMAX + 7)/8];
   unsigned int newMailCount;
   IMAP_CACHE cache[IMAP_CACHE_LEN];
-#ifdef USE_HCACHE
-  unsigned long uid_validity;
-#endif
-  
+  unsigned int uid_validity;
+  unsigned int uidnext;
+  body_cache_t *bcache;
+
   /* all folder flags - system flags AND keywords */
   LIST *flags;
 } IMAP_DATA;
@@ -195,21 +237,24 @@ typedef struct
 /* imap.c */
 int imap_create_mailbox (IMAP_DATA* idata, char* mailbox);
 int imap_rename_mailbox (IMAP_DATA* idata, IMAP_MBOX* mx, const char* newname);
-int imap_make_msg_set (IMAP_DATA* idata, BUFFER* buf, int flag, int changed);
+IMAP_STATUS* imap_mboxcache_get (IMAP_DATA* idata, const char* mbox);
+void imap_mboxcache_free (IMAP_DATA* idata);
+int imap_make_msg_set (IMAP_DATA* idata, BUFFER* buf, int flag, int changed,
+                       int invert);
 int imap_open_connection (IMAP_DATA* idata);
 IMAP_DATA* imap_conn_find (const ACCOUNT* account, int flags);
-int imap_parse_list_response(IMAP_DATA* idata, char** name, int* noselect,
-  int* noinferiors, char* delim);
 int imap_read_literal (FILE* fp, IMAP_DATA* idata, long bytes, progress_t*);
 void imap_expunge_mailbox (IMAP_DATA* idata);
 void imap_logout (IMAP_DATA* idata);
 int imap_sync_message (IMAP_DATA *idata, HEADER *hdr, BUFFER *cmd,
   int *err_continue);
+int imap_has_flag (LIST* flag_list, const char* flag);
 
 /* auth.c */
 int imap_authenticate (IMAP_DATA* idata);
 
 /* command.c */
+int imap_cmd_queue (IMAP_DATA* idata, const char* cmdstr);
 int imap_cmd_start (IMAP_DATA* idata, const char* cmd);
 int imap_cmd_step (IMAP_DATA* idata);
 void imap_cmd_finish (IMAP_DATA* idata);
@@ -221,6 +266,7 @@ void imap_add_keywords (char* s, HEADER* keywords, LIST* mailbox_flags, size_t s
 void imap_free_header_data (void** data);
 int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend);
 char* imap_set_flags (IMAP_DATA* idata, HEADER* h, char* s);
+int imap_cache_del (IMAP_DATA* idata, HEADER* h);
 
 /* util.c */
 int imap_continue (const char* msg, const char* resp);
@@ -231,6 +277,7 @@ char* imap_fix_path (IMAP_DATA* idata, char* mailbox, char* path,
   size_t plen);
 int imap_get_literal_count (const char* buf, long* bytes);
 char* imap_get_qualifier (char* buf);
+int imap_mxcmp (const char* mx1, const char* mx2);
 char* imap_next_word (char* s);
 time_t imap_parse_date (char* s);
 void imap_qualify_path (char *dest, size_t len, IMAP_MBOX *mx, char* path);
@@ -244,4 +291,9 @@ int imap_wordcasecmp(const char *a, const char *b);
 void imap_utf7_encode (char **s);
 void imap_utf7_decode (char **s);
 
+#if USE_HCACHE
+/* typedef size_t (*hcache_keylen_t)(const char* fn); */
+#define imap_hcache_keylen mutt_strlen
+#endif /* USE_HCACHE */
+
 #endif
