@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 1996-9 Brandon Long <blong@fiction.net>
- * Copyright (C) 1999-2005 Brendan Cully <brendan@kublai.com>
+ * Copyright (C) 1999-2006 Brendan Cully <brendan@kublai.com>
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -14,9 +14,8 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *
- * $Id: message.c,v 3.27 2005/09/07 07:44:05 brendan Exp $
  */ 
 
 /* message parsing/updating functions */
@@ -25,6 +24,7 @@
 # include "config.h"
 #endif
 
+#include <errno.h>
 #include <stdlib.h>
 #include <ctype.h>
 
@@ -37,19 +37,21 @@
 #include "pgp.h"
 #endif
 
+#if USE_HCACHE
+#include "hcache.h"
+#endif
+
+#include "bcache.h"
+
+static FILE* msg_cache_get (IMAP_DATA* idata, HEADER* h);
+static FILE* msg_cache_put (IMAP_DATA* idata, HEADER* h);
+
 static void flush_buffer(char* buf, size_t* len, CONNECTION* conn);
 static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf,
   FILE* fp);
-static int msg_has_flag (LIST* flag_list, const char* flag);
 static int msg_parse_fetch (IMAP_HEADER* h, char* s);
 static char* msg_parse_flags (IMAP_HEADER* h, char* s);
 
-#if USE_HCACHE
-static int msg_fetch_header_fetch (CONTEXT* ctx, IMAP_HEADER* h, char* buf,
-  FILE* fp);
-static size_t imap_hcache_keylen (const char *fn);
-#endif /* USE_HCACHE */
-
 /* imap_read_headers:
  * Changed to read many headers instead of just one. It will return the
  * msgno of the last message read. It will return a value other than
@@ -64,13 +66,17 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   char tempfile[_POSIX_PATH_MAX];
   int msgno;
   IMAP_HEADER h;
+  IMAP_STATUS* status;
   int rc, mfhrc, oldmsgcount;
   int fetchlast = 0;
+  int maxuid = 0;
   const char *want_headers = "DATE FROM SUBJECT TO CC MESSAGE-ID REFERENCES CONTENT-TYPE CONTENT-DESCRIPTION IN-REPLY-TO REPLY-TO LINES LIST-POST X-LABEL";
 
 #if USE_HCACHE
-  void *hc   = NULL;
-  unsigned long *uid_validity = NULL;
+  header_cache_t *hc = NULL;
+  unsigned int *uid_validity = NULL;
+  unsigned int *uidnext = NULL;
+  int evalhc = 0;
   char uid_buf[64];
 #endif /* USE_HCACHE */
 
@@ -113,91 +119,111 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
   idata->newMailCount = 0;
 
 #if USE_HCACHE
-  hc = mutt_hcache_open (HeaderCache, ctx->path);
+  if (!msgbegin)
+    hc = mutt_hcache_open (HeaderCache, ctx->path);
 
-  snprintf (buf, sizeof (buf),
-    "FETCH %d:%d (UID FLAGS)", msgbegin + 1, msgend + 1);
-  fetchlast = msgend + 1;
-
-  imap_cmd_start (idata, buf);
-
-  for (msgno = msgbegin; msgno <= msgend ; msgno++)
+  if (hc)
   {
-    if (ReadInc && (!msgno || ((msgno+1) % ReadInc == 0)))
-      mutt_message (_("Evaluating cache... [%d/%d]"), msgno + 1,
-        msgend + 1);
-
-    rewind (fp);
-    memset (&h, 0, sizeof (h));
-    h.data = safe_calloc (1, sizeof (IMAP_HEADER_DATA));
-    do
-    {
-      mfhrc = 0;
-
-      rc = imap_cmd_step (idata);
-      if (rc != IMAP_CMD_CONTINUE)
-	break;
-
-      if ((mfhrc = msg_fetch_header_fetch (idata->ctx, &h, idata->cmd.buf, fp)) == -1)
-	continue;
-      else if (mfhrc < 0)
-	break;
-
-      /* make sure we don't get remnants from older larger message headers */
-      fputs ("\n\n", fp);
-
-      sprintf(uid_buf, "/%u", h.data->uid); /* XXX --tg 21:41 04-07-11 */
-      uid_validity = (unsigned long *) mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
-
-      if (uid_validity != NULL && *uid_validity == idata->uid_validity)
-      {
-	ctx->hdrs[msgno] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
-	ctx->hdrs[msgno]->index = h.sid - 1;
-	if (h.sid != ctx->msgcount + 1)
-	  dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
-	/* messages which have not been expunged are ACTIVE (borrowed from mh 
-	 * folders) */
-	ctx->hdrs[msgno]->active = 1;
-	ctx->hdrs[msgno]->read = h.read;
-        ctx->hdrs[msgno]->old = h.old;
-        ctx->hdrs[msgno]->deleted = h.deleted;
-        ctx->hdrs[msgno]->flagged = h.flagged;
-        ctx->hdrs[msgno]->replied = h.replied;
-        ctx->hdrs[msgno]->changed = h.changed;
-        /*  ctx->hdrs[msgno]->received is restored from mutt_hcache_restore */
-        ctx->hdrs[msgno]->data = (void *) (h.data);
-        
-        ctx->msgcount++;
-      }
-      rewind (fp);
-
-      FREE(&uid_validity);
-
-    }
-    while ((rc != IMAP_CMD_OK) && ((mfhrc == -1) ||
-      ((msgno + 1) >= fetchlast)));
-
-    if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
-    {
-      imap_free_header_data ((void**) &h.data);
-      fclose (fp);
-      mutt_hcache_close (hc);
-      return -1;
-    }
+    uid_validity = mutt_hcache_fetch_raw (hc, "/UIDVALIDITY", imap_hcache_keylen);
+    uidnext = mutt_hcache_fetch_raw (hc, "/UIDNEXT", imap_hcache_keylen);
+    if (uid_validity && uidnext && *uid_validity == idata->uid_validity
+        && *uidnext > 0)
+      evalhc = 1;
+    FREE (&uid_validity);
   }
+  if (evalhc)
+  {
+    snprintf (buf, sizeof (buf),
+      "UID FETCH 1:%u (UID FLAGS)", *uidnext - 1);
+    FREE (&uidnext);
+  
+    imap_cmd_start (idata, buf);
+  
+    for (msgno = msgbegin; msgno <= msgend ; msgno++)
+    {
+      if (ReadInc && (!msgno || ((msgno+1) % ReadInc == 0)))
+        mutt_message (_("Evaluating cache... [%d/%d]"), msgno + 1,
+          msgend + 1);
+  
+      memset (&h, 0, sizeof (h));
+      h.data = safe_calloc (1, sizeof (IMAP_HEADER_DATA));
+      do
+      {
+        mfhrc = 0;
 
-  fetchlast = msgbegin;
+        rc = imap_cmd_step (idata);
+        if (rc != IMAP_CMD_CONTINUE)
+	{
+	  imap_free_header_data ((void**) &h.data);
+          break;
+	}
+
+        if ((mfhrc = msg_fetch_header (ctx, &h, idata->buf, NULL)) == -1)
+          continue;
+        else if (mfhrc < 0)
+	{
+	  imap_free_header_data ((void**) &h.data);
+          break;
+	}
+
+        sprintf(uid_buf, "/%u", h.data->uid); /* XXX --tg 21:41 04-07-11 */
+        uid_validity = (unsigned int*)mutt_hcache_fetch (hc, uid_buf, &imap_hcache_keylen);
+
+        if (uid_validity != NULL && *uid_validity == idata->uid_validity)
+        {
+  	  ctx->hdrs[msgno] = mutt_hcache_restore((unsigned char *) uid_validity, 0);
+  	  ctx->hdrs[msgno]->index = h.sid - 1;
+  	  if (h.sid != ctx->msgcount + 1)
+  	    dprint (1, (debugfile, "imap_read_headers: msgcount and sequence ID are inconsistent!"));
+  	  /* messages which have not been expunged are ACTIVE (borrowed from mh 
+  	   * folders) */
+  	  ctx->hdrs[msgno]->active = 1;
+          ctx->hdrs[msgno]->read = h.data->read;
+          ctx->hdrs[msgno]->old = h.data->old;
+          ctx->hdrs[msgno]->deleted = h.data->deleted;
+          ctx->hdrs[msgno]->flagged = h.data->flagged;
+          ctx->hdrs[msgno]->replied = h.data->replied;
+          ctx->hdrs[msgno]->changed = h.data->changed;
+          /*  ctx->hdrs[msgno]->received is restored from mutt_hcache_restore */
+          ctx->hdrs[msgno]->data = (void *) (h.data);
+
+          ctx->msgcount++;
+        }
+	else
+	  /* bad header in the cache, we'll have to refetch.
+	   * TODO: consider the possibility of a holey cache. */
+	  imap_free_header_data((void**) &h.data);
+  
+        FREE(&uid_validity);
+      }
+      while (rc != IMAP_CMD_OK && mfhrc == -1);
+      if (rc == IMAP_CMD_OK)
+        break;
+      if ((mfhrc < -1) || ((rc != IMAP_CMD_CONTINUE) && (rc != IMAP_CMD_OK)))
+      {
+        imap_free_header_data ((void**) &h.data);
+        fclose (fp);
+        mutt_hcache_close (hc);
+        return -1;
+      }
+    }
+  
+    fetchlast = msgbegin;
+  }
 #endif /* USE_HCACHE */
 
   for (msgno = msgbegin; msgno <= msgend ; msgno++)
   {
-    if (ReadInc && (!msgno || ((msgno+1) % ReadInc == 0)))
+    if (ctx->hdrs[msgno])
+    {
+      msgbegin++;
+      continue;
+    }
+
+    if (ReadInc && (msgno == msgbegin || ((msgno+1) % ReadInc == 0)))
       mutt_message (_("Fetching message headers... [%d/%d]"), msgno + 1,
         msgend + 1);
 
-    if (ctx->hdrs[msgno])
-      continue;
-
     if (msgno + 1 > fetchlast)
     {
       fetchlast = msgno + 1;
@@ -235,7 +261,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       if (rc != IMAP_CMD_CONTINUE)
 	break;
 
-      if ((mfhrc = msg_fetch_header (idata->ctx, &h, idata->cmd.buf, fp)) == -1)
+      if ((mfhrc = msg_fetch_header (ctx, &h, idata->buf, fp)) == -1)
 	continue;
       else if (mfhrc < 0)
 	break;
@@ -252,15 +278,18 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
       /* messages which have not been expunged are ACTIVE (borrowed from mh 
        * folders) */
       ctx->hdrs[msgno]->active = 1;
-      ctx->hdrs[msgno]->read = h.read;
-      ctx->hdrs[msgno]->old = h.old;
-      ctx->hdrs[msgno]->deleted = h.deleted;
-      ctx->hdrs[msgno]->flagged = h.flagged;
-      ctx->hdrs[msgno]->replied = h.replied;
-      ctx->hdrs[msgno]->changed = h.changed;
+      ctx->hdrs[msgno]->read = h.data->read;
+      ctx->hdrs[msgno]->old = h.data->old;
+      ctx->hdrs[msgno]->deleted = h.data->deleted;
+      ctx->hdrs[msgno]->flagged = h.data->flagged;
+      ctx->hdrs[msgno]->replied = h.data->replied;
+      ctx->hdrs[msgno]->changed = h.data->changed;
       ctx->hdrs[msgno]->received = h.received;
       ctx->hdrs[msgno]->data = (void *) (h.data);
 
+      if (maxuid < h.data->uid)
+        maxuid = h.data->uid;
+
       rewind (fp);
       /* NOTE: if Date: header is missing, mutt_read_rfc822_header depends
        *   on h.received being set */
@@ -288,7 +317,7 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
 #endif /* USE_HCACHE */
       return -1;
     }
-	
+
     /* in case we get new mail while fetching the headers */
     if (idata->reopen & IMAP_NEWMAIL_PENDING)
     {
@@ -300,7 +329,20 @@ int imap_read_headers (IMAP_DATA* idata, int msgbegin, int msgend)
     }
   }
 
+  if (maxuid && (status = imap_mboxcache_get (idata, idata->mailbox)))
+  status->uidnext = maxuid + 1;
+
 #if USE_HCACHE
+  mutt_hcache_store_raw (hc, "/UIDVALIDITY", &idata->uid_validity,
+                         sizeof (idata->uid_validity), imap_hcache_keylen);
+  if (maxuid && idata->uidnext < maxuid + 1)
+  {
+    dprint (2, (debugfile, "Overriding UIDNEXT: %u -> %u\n", idata->uidnext, maxuid + 1));
+    idata->uidnext = maxuid + 1;
+  }
+  if (idata->uidnext > 1)
+    mutt_hcache_store_raw (hc, "/UIDNEXT", &idata->uidnext,
+			   sizeof (idata->uidnext), imap_hcache_keylen);
   mutt_hcache_close (hc);
 #endif /* USE_HCACHE */
 
@@ -337,6 +379,15 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   idata = (IMAP_DATA*) ctx->data;
   h = ctx->hdrs[msgno];
 
+  if ((msg->fp = msg_cache_get (idata, h)))
+  {
+    if (HEADER_DATA(h)->parsed)
+      return 0;
+    else
+      goto parsemsg;
+  }
+
+  /* we still do some caching even if imap_cachedir is unset */
   /* see if we already have the message in our cache */
   cacheno = HEADER_DATA(h)->uid % IMAP_CACHE_LEN;
   cache = &idata->cache[cacheno];
@@ -357,13 +408,16 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   if (!isendwin())
     mutt_message _("Fetching message...");
 
-  cache->uid = HEADER_DATA(h)->uid;
-  mutt_mktemp (path);
-  cache->path = safe_strdup (path);
-  if (!(msg->fp = safe_fopen (path, "w+")))
+  if (!(msg->fp = msg_cache_put (idata, h)))
   {
-    FREE (&cache->path);
-    return -1;
+    cache->uid = HEADER_DATA(h)->uid;
+    mutt_mktemp (path);
+    cache->path = safe_strdup (path);
+    if (!(msg->fp = safe_fopen (path, "w+")))
+    {
+      FREE (&cache->path);
+      return -1;
+    }
   }
 
   /* mark this header as currently inactive so the command handler won't
@@ -382,7 +436,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
     if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
       break;
 
-    pc = idata->cmd.buf;
+    pc = idata->buf;
     pc = imap_next_word (pc);
     pc = imap_next_word (pc);
 
@@ -417,7 +471,7 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
 	  /* pick up trailing line */
 	  if ((rc = imap_cmd_step (idata)) != IMAP_CMD_CONTINUE)
 	    goto bail;
-	  pc = idata->cmd.buf;
+	  pc = idata->buf;
 
 	  fetched = 1;
 	}
@@ -448,9 +502,10 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
   if (rc != IMAP_CMD_OK)
     goto bail;
 
-  if (!fetched || !imap_code (idata->cmd.buf))
+  if (!fetched || !imap_code (idata->buf))
     goto bail;
-    
+
+parsemsg:
   /* Update the header information.  Previously, we only downloaded a
    * portion of the headers, those required for the main display.
    */
@@ -489,11 +544,13 @@ int imap_fetch_message (MESSAGE *msg, CONTEXT *ctx, int msgno)
 
   mutt_clear_error();
   rewind (msg->fp);
+  HEADER_DATA(h)->parsed = 1;
 
   return 0;
 
 bail:
   safe_fclose (&msg->fp);
+  imap_cache_del (idata, h);
   if (cache->path)
   {
     unlink (cache->path);
@@ -569,9 +626,9 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     char *pc;
 
     dprint (1, (debugfile, "imap_append_message(): command failed: %s\n",
-		idata->cmd.buf));
+		idata->buf));
 
-    pc = idata->cmd.buf + SEQLEN;
+    pc = idata->buf + SEQLEN;
     SKIPWS (pc);
     pc = imap_next_word (pc);
     mutt_error ("%s", pc);
@@ -605,13 +662,13 @@ int imap_append_message (CONTEXT *ctx, MESSAGE *msg)
     rc = imap_cmd_step (idata);
   while (rc == IMAP_CMD_CONTINUE);
 
-  if (!imap_code (idata->cmd.buf))
+  if (!imap_code (idata->buf))
   {
     char *pc;
 
     dprint (1, (debugfile, "imap_append_message(): command failed: %s\n",
-		idata->cmd.buf));
-    pc = idata->cmd.buf + SEQLEN;
+		idata->buf));
+    pc = idata->buf + SEQLEN;
     SKIPWS (pc);
     pc = imap_next_word (pc);
     mutt_error ("%s", pc);
@@ -699,7 +756,7 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
       }
     }
 
-    rc = imap_make_msg_set (idata, &cmd, M_TAG, 0);
+    rc = imap_make_msg_set (idata, &cmd, M_TAG, 0, 0);
     if (!rc)
     {
       dprint (1, (debugfile, "imap_copy_messages: No messages tagged\n"));
@@ -733,9 +790,9 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   if (rc == -2)
   {
     /* bail out if command failed for reasons other than nonexistent target */
-    if (ascii_strncasecmp (imap_get_qualifier (idata->cmd.buf), "[TRYCREATE]", 11))
+    if (ascii_strncasecmp (imap_get_qualifier (idata->buf), "[TRYCREATE]", 11))
     {
-      imap_error ("imap_copy_messages", idata->cmd.buf);
+      imap_error ("imap_copy_messages", idata->buf);
       goto fail;
     }
     dprint (2, (debugfile, "imap_copy_messages: server suggests TRYCREATE\n"));
@@ -753,7 +810,7 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   }
   if (rc != 0)
   {
-    imap_error ("imap_copy_messages", idata->cmd.buf);
+    imap_error ("imap_copy_messages", idata->buf);
     goto fail;
   }
 
@@ -794,6 +851,77 @@ int imap_copy_messages (CONTEXT* ctx, HEADER* h, char* dest, int delete)
   return -1;
 }
 
+static body_cache_t *msg_cache_open (IMAP_DATA *idata)
+{
+  char *s;
+  char *p = idata->mailbox;
+  char mailbox[_POSIX_PATH_MAX];
+  size_t mlen = sizeof (mailbox);
+
+  if (idata->bcache)
+    return idata->bcache;
+
+  mailbox[0] = '\0';
+
+  for (s = mailbox; p && *p && mlen; mlen--)
+  {
+    if (*p == idata->delim)
+    {
+      *s = '/';
+      /* simple way to avoid collisions with UIDs */
+      if (*(p + 1) >= '0' && *(p + 1) <= '9')
+      {
+	mlen--;
+	if (mlen)
+	  *++s = '_';
+      }
+    }
+    else
+      *s = *p;
+    *p++;
+    *s++;
+  }
+  *s = '\0';
+
+  return mutt_bcache_open (&idata->conn->account, mailbox);
+}
+
+static FILE* msg_cache_get (IMAP_DATA* idata, HEADER* h)
+{
+  char id[_POSIX_PATH_MAX];
+
+  if (!idata || !h)
+    return NULL;
+
+  idata->bcache = msg_cache_open (idata);
+  snprintf (id, sizeof (id), "%u-%u", idata->uid_validity, HEADER_DATA(h)->uid);
+  return mutt_bcache_get (idata->bcache, id);
+}
+
+static FILE* msg_cache_put (IMAP_DATA* idata, HEADER* h)
+{
+  char id[_POSIX_PATH_MAX];
+
+  if (!idata || !h)
+    return NULL;
+
+  idata->bcache = msg_cache_open (idata);
+  snprintf (id, sizeof (id), "%u-%u", idata->uid_validity, HEADER_DATA(h)->uid);
+  return mutt_bcache_put (idata->bcache, id);
+}
+
+int imap_cache_del (IMAP_DATA* idata, HEADER* h)
+{
+  char id[_POSIX_PATH_MAX];
+
+  if (!idata || !h)
+    return -1;
+
+  idata->bcache = msg_cache_open (idata);
+  snprintf (id, sizeof (id), "%u-%u", idata->uid_validity, HEADER_DATA(h)->uid);
+  return mutt_bcache_del (idata->bcache, id);
+}
+
 /* imap_add_keywords: concatenate custom IMAP tags to list, if they
  *   appear in the folder flags list. Why wouldn't they? */
 void imap_add_keywords (char* s, HEADER* h, LIST* mailbox_flags, size_t slen)
@@ -807,7 +935,7 @@ void imap_add_keywords (char* s, HEADER* h, LIST* mailbox_flags, size_t slen)
 
   while (keywords)
   {
-    if (msg_has_flag (mailbox_flags, keywords->data))
+    if (imap_has_flag (mailbox_flags, keywords->data))
     {
       safe_strcat (s, slen, keywords->data);
       safe_strcat (s, slen, " ");
@@ -822,7 +950,7 @@ void imap_free_header_data (void** data)
   /* this should be safe even if the list wasn't used */
   mutt_free_list (&(((IMAP_HEADER_DATA*) *data)->keywords));
 
-  FREE (data);
+  FREE (data);		/* __FREE_CHECKED__ */
 }
 
 /* imap_set_flags: fill out the message header according to the flags from
@@ -831,17 +959,16 @@ char* imap_set_flags (IMAP_DATA* idata, HEADER* h, char* s)
 {
   CONTEXT* ctx = idata->ctx;
   IMAP_HEADER newh;
+  IMAP_HEADER_DATA* hd;
   unsigned char readonly;
 
   memset (&newh, 0, sizeof (newh));
-  newh.data = safe_calloc (1, sizeof (IMAP_HEADER_DATA));
+  hd = h->data;
+  newh.data = hd;
 
   dprint (2, (debugfile, "imap_fetch_message: parsing FLAGS\n"));
   if ((s = msg_parse_flags (&newh, s)) == NULL)
-  {
-    FREE (&newh.data);
     return NULL;
-  }
   
   /* YAUH (yet another ugly hack): temporarily set context to
    * read-write even if it's read-only, so *server* updates of
@@ -850,12 +977,12 @@ char* imap_set_flags (IMAP_DATA* idata, HEADER* h, char* s)
   readonly = ctx->readonly;
   ctx->readonly = 0;
 	    
-  mutt_set_flag (ctx, h, M_NEW, !(newh.read || newh.old));
-  mutt_set_flag (ctx, h, M_OLD, newh.old);
-  mutt_set_flag (ctx, h, M_READ, newh.read);
-  mutt_set_flag (ctx, h, M_DELETE, newh.deleted);
-  mutt_set_flag (ctx, h, M_FLAG, newh.flagged);
-  mutt_set_flag (ctx, h, M_REPLIED, newh.replied);
+  mutt_set_flag (ctx, h, M_NEW, !(hd->read || hd->old));
+  mutt_set_flag (ctx, h, M_OLD, hd->old);
+  mutt_set_flag (ctx, h, M_READ, hd->read);
+  mutt_set_flag (ctx, h, M_DELETE, hd->deleted);
+  mutt_set_flag (ctx, h, M_FLAG, hd->flagged);
+  mutt_set_flag (ctx, h, M_REPLIED, hd->replied);
 
   /* this message is now definitively *not* changed (mutt_set_flag
    * marks things changed as a side-effect) */
@@ -863,10 +990,6 @@ char* imap_set_flags (IMAP_DATA* idata, HEADER* h, char* s)
   ctx->changed &= ~readonly;
   ctx->readonly = readonly;
 
-  mutt_free_list (&(HEADER_DATA(h)->keywords));
-  HEADER_DATA(h)->keywords = newh.data->keywords;
-  FREE(&newh.data);
-
   return s;
 }
 
@@ -904,7 +1027,7 @@ static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
 
   /* FIXME: current implementation - call msg_parse_fetch - if it returns -2,
    *   read header lines and call it again. Silly. */
-  if (msg_parse_fetch (h, buf) != -2)
+  if ((rc = msg_parse_fetch (h, buf) != -2) || !fp)
     return rc;
   
   if (imap_get_literal_count (buf, &bytes) == 0)
@@ -918,7 +1041,7 @@ static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
     if (imap_cmd_step (idata) != IMAP_CMD_CONTINUE)
       return rc;
   
-    if (msg_parse_fetch (h, idata->cmd.buf) == -1)
+    if (msg_parse_fetch (h, idata->buf) == -1)
       return rc;
   }
 
@@ -931,74 +1054,6 @@ static int msg_fetch_header (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
   return rc;
 }
 
-#if USE_HCACHE
-static size_t imap_hcache_keylen (const char *fn)
-{
-  return mutt_strlen(fn);
-}
-
-/* msg_fetch_header: import IMAP FETCH response into an IMAP_HEADER.
- *   Expects string beginning with * n FETCH.
- *   Returns:
- *      0 on success
- *     -1 if the string is not a fetch response
- *     -2 if the string is a corrupt fetch response */
-static int msg_fetch_header_fetch (CONTEXT* ctx, IMAP_HEADER* h, char* buf, FILE* fp)
-{
-  IMAP_DATA* idata;
-  int rc = -1; /* default now is that string isn't FETCH response*/
-
-  idata = (IMAP_DATA*) ctx->data;
-
-  if (buf[0] != '*')
-    return rc;
-
-  /* skip to message number */
-  buf = imap_next_word (buf);
-  h->sid = atoi (buf);
-
-  /* find FETCH tag */
-  buf = imap_next_word (buf);
-  if (ascii_strncasecmp ("FETCH", buf, 5))
-    return rc;
-
-  rc = -2; /* we've got a FETCH response, for better or worse */
-  if (!(buf = strchr (buf, '(')))
-    return rc;
-  buf++;
-
-  if (msg_parse_fetch (h, buf) < 0) {
-         return -2;
-  }
-
-  if (!(buf = strchr (buf, ')')))
-    return rc;
-  buf++;
-
-  return 0;
-}
-#endif /* USE_HCACHE */
-
-
-/* msg_has_flag: do a caseless comparison of the flag against a flag list,
- *   return 1 if found or flag list has '\*', 0 otherwise */
-static int msg_has_flag (LIST* flag_list, const char* flag)
-{
-  if (!flag_list)
-    return 0;
-
-  flag_list = flag_list->next;
-  while (flag_list)
-  {
-    if (!ascii_strncasecmp (flag_list->data, flag, strlen (flag_list->data)))
-      return 1;
-
-    flag_list = flag_list->next;
-  }
-
-  return 0;
-}
-
 /* msg_parse_fetch: handle headers returned from header fetch */
 static int msg_parse_fetch (IMAP_HEADER *h, char *s)
 {
@@ -1076,7 +1131,7 @@ static int msg_parse_fetch (IMAP_HEADER *h, char *s)
 /* msg_parse_flags: read a FLAGS token into an IMAP_HEADER */
 static char* msg_parse_flags (IMAP_HEADER* h, char* s)
 {
-  int recent = 0;
+  IMAP_HEADER_DATA* hd = h->data;
 
   /* sanity-check string */
   if (ascii_strncasecmp ("FLAGS", s, 5) != 0)
@@ -1095,33 +1150,38 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
   }
   s++;
 
+  mutt_free_list (&hd->keywords);
+  hd->deleted = hd->flagged = hd->replied = hd->read = hd->old = 0;
+
   /* start parsing */
   while (*s && *s != ')')
   {
     if (ascii_strncasecmp ("\\deleted", s, 8) == 0)
     {
       s += 8;
-      h->deleted = 1;
+      hd->deleted = 1;
     }
     else if (ascii_strncasecmp ("\\flagged", s, 8) == 0)
     {
       s += 8;
-      h->flagged = 1;
+      hd->flagged = 1;
     }
     else if (ascii_strncasecmp ("\\answered", s, 9) == 0)
     {
       s += 9;
-      h->replied = 1;
+      hd->replied = 1;
     }
     else if (ascii_strncasecmp ("\\seen", s, 5) == 0)
     {
       s += 5;
-      h->read = 1;
+      hd->read = 1;
     }
-    else if (ascii_strncasecmp ("\\recent", s, 5) == 0)
-    {
+    else if (ascii_strncasecmp ("\\recent", s, 7) == 0)
       s += 7;
-      recent = 1;
+    else if (ascii_strncasecmp ("old", s, 3) == 0)
+    {
+      s += 3;
+      hd->old = 1;
     }
     else
     {
@@ -1129,14 +1189,14 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
       char ctmp;
       char* flag_word = s;
 
-      if (!h->data->keywords)
-        h->data->keywords = mutt_new_list ();
+      if (!hd->keywords)
+        hd->keywords = mutt_new_list ();
 
       while (*s && !ISSPACE (*s) && *s != ')')
         s++;
       ctmp = *s;
       *s = '\0';
-      mutt_add_list (h->data->keywords, flag_word);
+      mutt_add_list (hd->keywords, flag_word);
       *s = ctmp;
     }
     SKIPWS(s);
@@ -1144,12 +1204,7 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
 
   /* wrap up, or note bad flags response */
   if (*s == ')')
-  {
-    /* if a message is neither seen nor recent, it is OLD. */
-    if (option (OPTMARKOLD) && !recent && !(h->read))
-      h->old = 1;
     s++;
-  }
   else
   {
     dprint (1, (debugfile,
@@ -1163,6 +1218,6 @@ static char* msg_parse_flags (IMAP_HEADER* h, char* s)
 static void flush_buffer(char *buf, size_t *len, CONNECTION *conn)
 {
   buf[*len] = '\0';
-  mutt_socket_write(conn, buf);
+  mutt_socket_write_n(conn, buf, *len);
   *len = 0;
 }
