@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /*
@@ -32,6 +32,10 @@
 #include "copy.h"
 #include "buffy.h"
 #include "sort.h"
+#include "account.h"
+#if USE_HCACHE
+#include "hcache.h"
+#endif
 
 #include <sys/stat.h>
 #include <sys/types.h>
@@ -219,7 +223,7 @@ static int mh_mkstemp (CONTEXT * dest, FILE ** fp, char **tgt)
 
   if ((*fp = fdopen (fd, "w")) == NULL)
   {
-    FREE (tgt);
+    FREE (tgt);		/* __FREE_CHECKED__ */
     close (fd);
     unlink (path);
     return (-1);
@@ -485,7 +489,7 @@ static void maildir_free_entry (struct maildir **md)
   if ((*md)->h)
     mutt_free_header (&(*md)->h);
 
-  FREE (md);
+  FREE (md);		/* __FREE_CHECKED__ */
 }
 
 static void maildir_free_maildir (struct maildir **md)
@@ -638,11 +642,18 @@ static HEADER *maildir_parse_message (int magic, const char *fname,
 
 static int maildir_parse_entry (CONTEXT * ctx, struct maildir ***last,
 				const char *subdir, const char *fname,
-				int *count, int is_old, ino_t inode)
+				int *count, int is_old, ino_t inode
+#if USE_HCACHE
+				, header_cache_t *hc
+#endif
+			       )
 {
   struct maildir *entry;
   HEADER *h = NULL;
   char buf[_POSIX_PATH_MAX];
+#if USE_HCACHE
+  void *data;
+#endif
 
   if (subdir)
     snprintf (buf, sizeof (buf), "%s/%s/%s", ctx->path, subdir, fname);
@@ -650,7 +661,14 @@ static int maildir_parse_entry (CONTEXT * ctx, struct maildir ***last,
     snprintf (buf, sizeof (buf), "%s/%s", ctx->path, fname);
 
   if (ctx->magic == M_MH)
+  {
+#ifdef USE_HCACHE
+    if (hc && (data = mutt_hcache_fetch (hc, fname, strlen)))
+      h = mutt_hcache_restore ((unsigned char *) data, NULL);
+    else
+#endif
     h = maildir_parse_message (ctx->magic, buf, is_old, NULL);
+  }
   else
   {
     h = mutt_new_header ();
@@ -714,6 +732,9 @@ static int maildir_parse_dir (CONTEXT * ctx, struct maildir ***last,
   struct dirent *de;
   char buf[_POSIX_PATH_MAX];
   int is_old = 0;
+#ifdef USE_HCACHE
+  header_cache_t *hc = NULL;
+#endif
 
   if (subdir)
   {
@@ -726,6 +747,11 @@ static int maildir_parse_dir (CONTEXT * ctx, struct maildir ***last,
   if ((dirp = opendir (buf)) == NULL)
     return -1;
 
+#ifdef USE_HCACHE
+  if (ctx && ctx->magic == M_MH)
+    hc = mutt_hcache_open (HeaderCache, ctx->path);
+#endif
+
   while ((de = readdir (dirp)) != NULL)
   {
 
@@ -739,10 +765,23 @@ static int maildir_parse_dir (CONTEXT * ctx, struct maildir ***last,
 	    (debugfile, "%s:%d: parsing %s\n", __FILE__, __LINE__,
 	     de->d_name));
     maildir_parse_entry (ctx, last, subdir, de->d_name, count, is_old, 
-			 de->d_ino);
+#if HAVE_DIRENT_D_INO
+			 de->d_ino
+#else
+			 0
+#endif
+#if USE_HCACHE
+			 , hc
+#endif
+			 );
   }
 
   closedir (dirp);
+
+#if USE_HCACHE
+  mutt_hcache_close (hc);
+#endif
+
   return 0;
 }
 
@@ -905,7 +944,7 @@ void maildir_delayed_parsing (CONTEXT * ctx, struct maildir *md)
   int count;
 
 #if USE_HCACHE
-  void *hc = NULL;
+  header_cache_t *hc = NULL;
   void *data;
   struct timeval *when = NULL;
   struct stat lastchanged;
@@ -1458,7 +1497,7 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
   char path[_POSIX_PATH_MAX], tmp[_POSIX_PATH_MAX];
   int i, j;
 #if USE_HCACHE
-  void *hc = NULL;
+  header_cache_t *hc = NULL;
 #endif /* USE_HCACHE */
 
   if (ctx->magic == M_MH)
@@ -1470,7 +1509,7 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
     return i;
 
 #if USE_HCACHE
-  if (ctx->magic == M_MAILDIR)
+  if (ctx->magic == M_MAILDIR || ctx->magic == M_MH)
     hc = mutt_hcache_open(HeaderCache, ctx->path);
 #endif /* USE_HCACHE */
 
@@ -1486,6 +1525,8 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
 #if USE_HCACHE
         if (ctx->magic == M_MAILDIR)
           mutt_hcache_delete (hc, ctx->hdrs[i]->path + 3, &maildir_hcache_keylen);
+	else if (ctx->magic == M_MH)
+	  mutt_hcache_delete (hc, ctx->hdrs[i]->path, strlen);
 #endif /* USE_HCACHE */
 	unlink (path);
       }
@@ -1521,7 +1562,7 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
   }
 
 #if USE_HCACHE
-  if (ctx->magic == M_MAILDIR)
+  if (ctx->magic == M_MAILDIR || ctx->magic == M_MH)
     mutt_hcache_close (hc);
 #endif /* USE_HCACHE */
 
@@ -1548,7 +1589,7 @@ int mh_sync_mailbox (CONTEXT * ctx, int *index_hint)
 
 err:
 #if USE_HCACHE
-  if (ctx->magic == M_MAILDIR)
+  if (ctx->magic == M_MAILDIR || ctx->magic == M_MH)
     mutt_hcache_close (hc);
 #endif /* USE_HCACHE */
   return -1;
