@@ -15,7 +15,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #ifndef MUTT_H
@@ -238,6 +238,7 @@ enum
   M_CRYPT_ENCRYPT,
   M_PGP_KEY,
   M_XLABEL,
+  M_MIMEATTACH,
   
   /* Options for Mailcap lookup */
   M_EDIT,
@@ -247,7 +248,7 @@ enum
 
   /* options for socket code */
   M_NEW_SOCKET,
-#ifdef USE_SSL
+#ifdef USE_SSL_OPENSSL
   M_NEW_SSL_SOCKET,
 #endif
 
@@ -288,7 +289,7 @@ enum
   OPT_QUIT,
   OPT_REPLYTO,
   OPT_RECALL,
-#if defined(USE_SSL) || defined(USE_GNUTLS)
+#if defined(USE_SSL)
   OPT_SSLSTARTTLS,
 #endif
   OPT_SUBJECT,
@@ -353,6 +354,9 @@ enum
   OPTFORWQUOTE,
 #if USE_HCACHE
   OPTHCACHEVERIFY,
+#if HAVE_QDBM
+  OPTHCACHECOMPRESS,
+#endif /* HAVE_QDBM */
 #endif
   OPTHDRS,
   OPTHEADER,
@@ -366,20 +370,21 @@ enum
   OPTIGNORELISTREPLYTO,
 #ifdef USE_IMAP
   OPTIMAPCHECKSUBSCRIBED,
+  OPTIMAPIDLE,
   OPTIMAPLSUB,
   OPTIMAPPASSIVE,
   OPTIMAPPEEK,
   OPTIMAPSERVERNOISE,
 #endif
-#if defined(USE_SSL) || defined(USE_GNUTLS)
-# ifndef USE_GNUTLS
+#if defined(USE_SSL)
+# ifndef USE_SSL_GNUTLS
   OPTSSLSYSTEMCERTS,
   OPTSSLV2,
-# endif /* USE_GNUTLS */
+# endif /* USE_SSL_GNUTLS */
   OPTSSLV3,
   OPTTLSV1,
   OPTSSLFORCETLS,
-#endif /* defined(USE_SSL) || defined(USE_GNUTLS) */
+#endif /* defined(USE_SSL) */
   OPTIMPLICITAUTOVIEW,
   OPTINCLUDEONLYFIRST,
   OPTKEEPFLAGGED,
@@ -623,8 +628,8 @@ typedef struct body
 				 * where we need to send the headers of the
 				 * attachment
 				 */
-  long offset;                  /* offset where the actual data begins */
-  long length;                  /* length (in bytes) of attachment */
+  LOFF_T offset;                /* offset where the actual data begins */
+  LOFF_T length;                /* length (in bytes) of attachment */
   char *filename;               /* when sending a message, this is the file
 				 * to which this structure refers
 				 */
@@ -643,7 +648,9 @@ typedef struct body
   struct header *hdr;		/* header information for message/rfc822 */
 
   struct attachptr *aptr;	/* Menu information, used in recvattach.c */
-  
+
+  signed short attach_count;
+
   time_t stamp;			/* time stamp of last
 				 * encoding update.
 				 */
@@ -681,6 +688,7 @@ typedef struct body
   unsigned int badsig : 1;	/* bad cryptographic signature (needed to check encrypted s/mime-signatures) */
 
   unsigned int collapsed : 1;	/* used by recvattach */
+  unsigned int attach_qualifies : 1;
 
 } BODY;
 
@@ -689,7 +697,7 @@ typedef struct header
   unsigned int security : 11;  /* bit 0-6: flags, bit 7,8: application.
 				 see: crypt.h pgplib.h, smime.h */
 
-  unsigned int mime : 1;    		/* has a Mime-Version header? */
+  unsigned int mime : 1;    		/* has a MIME-Version header? */
   unsigned int flagged : 1; 		/* marked important? */
   unsigned int tagged : 1;
   unsigned int deleted : 1;
@@ -719,6 +727,9 @@ typedef struct header
   unsigned int searched : 1;
   unsigned int matched : 1;
 
+  /* tells whether the attachment count is valid */
+  unsigned int attach_valid : 1;
+
   /* the following are used to support collapsing threads  */
   unsigned int collapsed : 1; 	/* is this message part of a collapsed thread? */
   unsigned int limited : 1;   	/* is this message in a limited view?  */
@@ -730,7 +741,7 @@ typedef struct header
 
   time_t date_sent;     	/* time when the message was sent (UTC) */
   time_t received;      	/* time when the message was placed in the mailbox */
-  long offset;          	/* where in the stream does this message begin? */
+  LOFF_T offset;          	/* where in the stream does this message begin? */
   int lines;			/* how many lines in the body of this message? */
   int index;			/* the absolute (unsorted) message number */
   int msgno;			/* number displayed to the user */
@@ -743,6 +754,9 @@ typedef struct header
   char *tree;           	/* character string to print thread tree */
   struct thread *thread;
 
+  /* Number of qualifying attachments in message, if attach_valid */
+  short attach_total;
+
 #ifdef MIXMASTER
   LIST *chain;
 #endif
@@ -784,18 +798,36 @@ typedef enum {
   M_MATCH_FULL_ADDRESS = 1
 } pattern_exec_flag;
 
+typedef struct group_t
+{
+  ADDRESS *as;
+  RX_LIST *rs;
+  char *name;
+} group_t;
+
+typedef struct group_context_t
+{
+  group_t *g;
+  struct group_context_t *next;
+} group_context_t;
+
 typedef struct pattern_t
 {
   short op;
   unsigned int not : 1;
   unsigned int alladdr : 1;
   unsigned int stringmatch : 1;
+  unsigned int groupmatch : 1;
   int min;
   int max;
   struct pattern_t *next;
   struct pattern_t *child;		/* arguments to logical op */
-  char *str;
-  regex_t *rx;
+  union 
+  {
+    regex_t *rx;
+    group_t *g;
+    char *str;
+  } p;
 } pattern_t;
 
 typedef struct
@@ -880,6 +912,17 @@ void state_attach_puts (const char *, STATE *);
 void state_prefix_putc (char, STATE *);
 int  state_printf(STATE *, const char *, ...);
 
+/* for attachment counter */
+typedef struct
+{
+  char   *major;
+  int     major_int;
+  char   *minor;
+  regex_t minor_rx;
+} ATTACH_MATCH;
+
+#define M_PARTS_TOPLEVEL	(1<<0)	/* is the top-level part */
+
 #include "ascii.h"
 #include "protos.h"
 #include "lib.h"
