@@ -15,7 +15,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -26,7 +26,7 @@
 #include "globals.h"
 #include "mutt_socket.h"
 #include "mutt_tunnel.h"
-#if defined(USE_SSL) || defined(USE_GNUTLS)
+#if defined(USE_SSL)
 # include "mutt_ssl.h"
 #endif
 
@@ -39,6 +39,9 @@
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/socket.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #include <string.h>
 #include <errno.h>
 
@@ -97,10 +100,9 @@ int mutt_socket_read (CONNECTION* conn, char* buf, size_t len)
   return rc;
 }
 
-int mutt_socket_write_d (CONNECTION *conn, const char *buf, int dbg)
+int mutt_socket_write_d (CONNECTION *conn, const char *buf, int len, int dbg)
 {
   int rc;
-  int len;
 
   dprint (dbg, (debugfile,"> %s", buf));
 
@@ -110,7 +112,8 @@ int mutt_socket_write_d (CONNECTION *conn, const char *buf, int dbg)
     return -1;
   }
 
-  len = mutt_strlen (buf);
+  if (len < 0)
+    len = mutt_strlen (buf);
   if ((rc = conn->conn_write (conn, buf, len)) < 0)
   {
     dprint (1, (debugfile,
@@ -129,6 +132,21 @@ int mutt_socket_write_d (CONNECTION *conn, const char *buf, int dbg)
   return rc;
 }
 
+/* poll whether reads would block.
+ *   Returns: >0 if there is data to read,
+ *            0 if a read would block,
+ *            -1 if this connection doesn't support polling */
+int mutt_socket_poll (CONNECTION* conn)
+{
+  if (conn->bufpos < conn->available)
+    return conn->available - conn->bufpos;
+
+  if (conn->conn_poll)
+    return conn->conn_poll (conn);
+
+  return -1;
+}
+
 /* simple read buffering to speed things up. */
 int mutt_socket_readchar (CONNECTION *conn, char *c)
 {
@@ -257,10 +275,8 @@ CONNECTION* mutt_conn_find (const CONNECTION* start, const ACCOUNT* account)
     mutt_tunnel_socket_setup (conn);
   else if (account->flags & M_ACCT_SSL) 
   {
-#ifdef USE_SSL
-    ssl_socket_setup (conn);
-#elif USE_GNUTLS
-    if (mutt_gnutls_socket_setup (conn) < 0)
+#if defined(USE_SSL)
+    if (mutt_ssl_socket_setup (conn) < 0)
     {
       mutt_socket_free (conn);
       return NULL;
@@ -279,6 +295,7 @@ CONNECTION* mutt_conn_find (const CONNECTION* start, const ACCOUNT* account)
     conn->conn_write = raw_socket_write;
     conn->conn_open = raw_socket_open;
     conn->conn_close = raw_socket_close;
+    conn->conn_poll = raw_socket_poll;
   }
 
   return conn;
@@ -390,6 +407,20 @@ int raw_socket_write (CONNECTION* conn, const char* buf, size_t count)
   return rc;
 }
 
+int raw_socket_poll (CONNECTION* conn)
+{
+  fd_set rfds;
+  struct timeval tv = { 0, 0 };
+
+  if (conn->fd < 0)
+    return -1;
+
+  FD_ZERO (&rfds);
+  FD_SET (conn->fd, &rfds);
+  
+  return select (conn->fd + 1, &rfds, NULL, NULL, &tv);
+}
+
 int raw_socket_open (CONNECTION* conn)
 {
   int rc;
