@@ -14,7 +14,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -900,15 +900,11 @@ int mx_close_mailbox (CONTEXT *ctx, int *index_hint)
     }
   }
 
-#ifdef USE_IMAP
-  /* IMAP servers manage the OLD flag themselves */
-  if (ctx->magic != M_IMAP)
-#endif
   if (option (OPTMARKOLD))
   {
     for (i = 0; i < ctx->msgcount; i++)
     {
-      if (!ctx->hdrs[i]->deleted && !ctx->hdrs[i]->old)
+      if (!ctx->hdrs[i]->deleted && !ctx->hdrs[i]->old && !ctx->hdrs[i]->read)
 	mutt_set_flag (ctx, ctx->hdrs[i], M_OLD, 1);
     }
   }
@@ -1530,7 +1526,7 @@ int mx_close_message (MESSAGE **msg)
     FREE (&(*msg)->path);
   }
 
-  FREE (msg);
+  FREE (msg);		/* __FREE_CHECKED__ */
   return (r);
 }
 
