@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -300,6 +300,10 @@ int mutt_check_mime_type (const char *s)
     return TYPEVIDEO;
   else if (ascii_strcasecmp ("model", s) == 0)
     return TYPEMODEL;
+  else if (ascii_strcasecmp ("*", s) == 0)
+    return TYPEANY;
+  else if (ascii_strcasecmp (".*", s) == 0)
+    return TYPEANY;
   else
     return TYPEOTHER;
 }
@@ -427,7 +431,7 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
   char *line = safe_malloc (LONG_STRING);
   size_t linelen = LONG_STRING;
   
-  p->hdr_offset  = ftell(fp);
+  p->hdr_offset  = ftello (fp);
 
   p->encoding    = ENC7BIT; /* default from RFC1521 */
   p->type        = digest ? TYPEMESSAGE : TYPETEXT;
@@ -484,7 +488,7 @@ BODY *mutt_read_mime_header (FILE *fp, int digest)
     }
 #endif
   }
-  p->offset = ftell (fp); /* Mark the start of the real data */
+  p->offset = ftello (fp); /* Mark the start of the real data */
   if (p->type == TYPETEXT && !p->subtype)
     p->subtype = safe_strdup ("plain");
   else if (p->type == TYPEMESSAGE && !p->subtype)
@@ -509,7 +513,7 @@ void mutt_parse_part (FILE *fp, BODY *b)
 #endif
           bound = mutt_get_parameter ("boundary", b->parameter);
 
-      fseek (fp, b->offset, SEEK_SET);
+      fseeko (fp, b->offset, SEEK_SET);
       b->parts =  mutt_parse_multipart (fp, bound, 
 					b->offset + b->length,
 					ascii_strcasecmp ("digest", b->subtype) == 0);
@@ -518,7 +522,7 @@ void mutt_parse_part (FILE *fp, BODY *b)
     case TYPEMESSAGE:
       if (b->subtype)
       {
-	fseek (fp, b->offset, SEEK_SET);
+	fseeko (fp, b->offset, SEEK_SET);
 	if (mutt_is_message_type(b->type, b->subtype))
 	  b->parts = mutt_parse_messageRFC822 (fp, b);
 	else if (ascii_strcasecmp (b->subtype, "external-body") == 0)
@@ -556,7 +560,7 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
   BODY *msg;
 
   parent->hdr = mutt_new_header ();
-  parent->hdr->offset = ftell (fp);
+  parent->hdr->offset = ftello (fp);
   parent->hdr->env = mutt_read_rfc822_header (fp, parent->hdr, 0, 0);
   msg = parent->hdr->content;
 
@@ -586,7 +590,7 @@ BODY *mutt_parse_messageRFC822 (FILE *fp, BODY *parent)
  *	digest		1 if reading a multipart/digest, 0 otherwise
  */
 
-BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int digest)
+BODY *mutt_parse_multipart (FILE *fp, const char *boundary, LOFF_T end_off, int digest)
 {
 #ifdef SUN_ATTACHMENT
   int lines;
@@ -604,7 +608,7 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
   }
 
   blen = mutt_strlen (boundary);
-  while (ftell (fp) < end_off && fgets (buffer, LONG_STRING, fp) != NULL)
+  while (ftello (fp) < end_off && fgets (buffer, LONG_STRING, fp) != NULL)
   {
     len = mutt_strlen (buffer);
 
@@ -615,9 +619,9 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
     {
       if (last)
       {
-	last->length = ftell (fp) - last->offset - len - 1 - crlf;
+	last->length = ftello (fp) - last->offset - len - 1 - crlf;
 	if (last->parts && last->parts->length == 0)
-	  last->parts->length = ftell (fp) - last->parts->offset - len - 1 - crlf;
+	  last->parts->length = ftello (fp) - last->parts->offset - len - 1 - crlf;
 	/* if the body is empty, we can end up with a -1 length */
 	if (last->length < 0)
 	  last->length = 0;
@@ -641,7 +645,7 @@ BODY *mutt_parse_multipart (FILE *fp, const char *boundary, long end_off, int di
         if (mutt_get_parameter ("content-lines", new->parameter)) {
 	  for (lines = atoi(mutt_get_parameter ("content-lines", new->parameter));
 	       lines; lines-- )
-	     if (ftell (fp) >= end_off || fgets (buffer, LONG_STRING, fp) == NULL)
+	     if (ftello (fp) >= end_off || fgets (buffer, LONG_STRING, fp) == NULL)
 	       break;
 	}
 #endif
@@ -925,21 +929,26 @@ void mutt_parse_mime_message (CONTEXT *ctx, HEADER *cur)
 {
   MESSAGE *msg;
 
-  if (cur->content->type != TYPEMESSAGE && cur->content->type != TYPEMULTIPART)
-    return; /* nothing to do */
+  do {
+    if (cur->content->type != TYPEMESSAGE &&
+        cur->content->type != TYPEMULTIPART)
+      break; /* nothing to do */
 
-  if (cur->content->parts)
-    return; /* The message was parsed earlier. */
+    if (cur->content->parts)
+      break; /* The message was parsed earlier. */
 
-  if ((msg = mx_open_message (ctx, cur->msgno)))
-  {
-    mutt_parse_part (msg->fp, cur->content);
+    if ((msg = mx_open_message (ctx, cur->msgno)))
+    {
+      mutt_parse_part (msg->fp, cur->content);
 
-    if (WithCrypto)
-      cur->security = crypt_query (cur->content);
+      if (WithCrypto)
+        cur->security = crypt_query (cur->content);
 
-    mx_close_message (&msg);
-  }
+      mx_close_message (&msg);
+    }
+  } while (0);
+
+  cur->attach_valid = 0;
 }
 
 int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short user_hdrs, short weed,
@@ -1105,7 +1114,7 @@ int mutt_parse_rfc822_line (ENVELOPE *e, HEADER *hdr, char *line, char *p, short
     }
     else if (!ascii_strcasecmp (line + 1, "essage-id"))
     {
-      /* We add a new "Message-Id:" when building a message */
+      /* We add a new "Message-ID:" when building a message */
       FREE (&e->message_id);
       e->message_id = extract_message_id (p);
       matched = 1;
@@ -1294,7 +1303,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
   LIST *last = NULL;
   char *line = safe_malloc (LONG_STRING);
   char *p;
-  long loc;
+  LOFF_T loc;
   int matched;
   size_t linelen = LONG_STRING;
   char buf[LONG_STRING+1];
@@ -1316,7 +1325,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
     }
   }
 
-  while ((loc = ftell (f)),
+  while ((loc = ftello (f)),
 	  *(line = mutt_read_rfc822_line (f, line, &linelen)) != 0)
   {
     matched = 0;
@@ -1337,7 +1346,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
 	continue;
       }
 
-      fseek (f, loc, 0);
+      fseeko (f, loc, 0);
       break; /* end of header */
     }
 
@@ -1399,7 +1408,7 @@ ENVELOPE *mutt_read_rfc822_header (FILE *f, HEADER *hdr, short user_hdrs,
   if (hdr)
   {
     hdr->content->hdr_offset = hdr->offset;
-    hdr->content->offset = ftell (f);
+    hdr->content->offset = ftello (f);
 
     /* do RFC2047 decoding */
     rfc2047_decode_adrlist (e->from);
@@ -1457,3 +1466,159 @@ ADDRESS *mutt_parse_adrlist (ADDRESS *p, const char *s)
   
   return p;
 }
+
+/* Compares mime types to the ok and except lists */
+int count_body_parts_check(LIST **checklist, BODY *b, int dflt)
+{
+  LIST *type;
+  ATTACH_MATCH *a;
+
+  /* If list is null, use default behavior. */
+  if (! *checklist)
+  {
+    /*return dflt;*/
+    return 0;
+  }
+
+  for (type = *checklist; type; type = type->next)
+  {
+    a = (ATTACH_MATCH *)type->data;
+    dprint(5, (debugfile, "cbpc: %s %d/%s ?? %s/%s [%d]... ",
+		dflt ? "[OK]   " : "[EXCL] ",
+		b->type, b->subtype, a->major, a->minor, a->major_int));
+    if ((a->major_int == TYPEANY || a->major_int == b->type) &&
+	!regexec(&a->minor_rx, b->subtype, 0, NULL, 0))
+    {
+      dprint(5, (debugfile, "yes\n"));
+      return 1;
+    }
+    else
+    {
+      dprint(5, (debugfile, "no\n"));
+    }
+  }
+
+  return 0;
+}
+
+#define AT_COUNT(why)   { shallcount = 1; }
+#define AT_NOCOUNT(why) { shallcount = 0; }
+
+int count_body_parts (BODY *body, int flags)
+{
+  int count = 0;
+  int shallcount, shallrecurse;
+  BODY *bp;
+
+  if (body == NULL)
+    return 0;
+
+  for (bp = body; bp != NULL; bp = bp->next)
+  {
+    /* Initial disposition is to count and not to recurse this part. */
+    AT_COUNT("default");
+    shallrecurse = 0;
+
+    dprint(5, (debugfile, "bp: desc=\"%s\"; fn=\"%s\", type=\"%d/%s\"\n",
+	   bp->description ? bp->description : ("none"),
+	   bp->filename ? bp->filename :
+			bp->d_filename ? bp->d_filename : "(none)",
+	   bp->type, bp->subtype ? bp->subtype : "*"));
+
+    if (bp->type == TYPEMESSAGE)
+    {
+      shallrecurse = 1;
+
+      /* If it's an external body pointer, don't recurse it. */
+      if (!ascii_strcasecmp (bp->subtype, "external-body"))
+	shallrecurse = 0;
+
+      /* Don't count containers if they're top-level. */
+      if (flags & M_PARTS_TOPLEVEL)
+	AT_NOCOUNT("top-level message/*");
+    }
+    else if (bp->type == TYPEMULTIPART)
+    {
+      /* Always recurse multiparts, except multipart/alternative. */
+      shallrecurse = 1;
+      if (!ascii_strcasecmp(bp->subtype, "alternative"))
+        shallrecurse = 0;
+
+      /* Don't count containers if they're top-level. */
+      if (flags & M_PARTS_TOPLEVEL)
+	AT_NOCOUNT("top-level multipart");
+    }
+
+    if (bp->disposition == DISPINLINE &&
+        bp->type != TYPEMULTIPART && bp->type != TYPEMESSAGE && bp == body)
+      AT_NOCOUNT("ignore fundamental inlines");
+
+    /* If this body isn't scheduled for enumeration already, don't bother
+     * profiling it further.
+     */
+    if (shallcount)
+    {
+      /* Turn off shallcount if message type is not in ok list,
+       * or if it is in except list. Check is done separately for
+       * inlines vs. attachments.
+       */
+
+      if (bp->disposition == DISPATTACH)
+      {
+        if (!count_body_parts_check(&AttachAllow, bp, 1))
+	  AT_NOCOUNT("attach not allowed");
+        if (count_body_parts_check(&AttachExclude, bp, 0))
+	  AT_NOCOUNT("attach excluded");
+      }
+      else
+      {
+        if (!count_body_parts_check(&InlineAllow, bp, 1))
+	  AT_NOCOUNT("inline not allowed");
+        if (count_body_parts_check(&InlineExclude, bp, 0))
+	  AT_NOCOUNT("excluded");
+      }
+    }
+
+    if (shallcount)
+      count++;
+    bp->attach_qualifies = shallcount ? 1 : 0;
+
+    dprint(5, (debugfile, "cbp: %08x shallcount = %d\n", (unsigned int)bp, shallcount));
+
+    if (shallrecurse)
+    {
+      dprint(5, (debugfile, "cbp: %08x pre count = %d\n", (unsigned int)bp, count));
+      bp->attach_count = count_body_parts(bp->parts, flags & ~M_PARTS_TOPLEVEL);
+      count += bp->attach_count;
+      dprint(5, (debugfile, "cbp: %08x post count = %d\n", (unsigned int)bp, count));
+    }
+  }
+
+  dprint(5, (debugfile, "bp: return %d\n", count < 0 ? 0 : count));
+  return count < 0 ? 0 : count;
+}
+
+int mutt_count_body_parts (CONTEXT *ctx, HEADER *hdr)
+{
+  short keep_parts = 0;
+
+  if (hdr->attach_valid)
+    return hdr->attach_total;
+  
+  if (hdr->content->parts)
+    keep_parts = 1;
+  else
+    mutt_parse_mime_message (ctx, hdr);
+  
+  if (AttachAllow || AttachExclude || InlineAllow || InlineExclude)
+    hdr->attach_total = count_body_parts(hdr->content, M_PARTS_TOPLEVEL);
+  else
+    hdr->attach_total = 0;
+
+  hdr->attach_valid = 1;
+  
+  if (!keep_parts)
+    mutt_free_body (&hdr->content->parts);
+  
+  return hdr->attach_total;
+}
