@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 1996-2000 Michael R. Elkins <me@mutt.org>, and others
+ * Copyright (C) 1996-2000,2006 Michael R. Elkins <me@mutt.org>, and others
  * 
  *     This program is free software; you can redistribute it and/or modify
  *     it under the terms of the GNU General Public License as published by
@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 #if HAVE_CONFIG_H
@@ -91,6 +91,7 @@ Flags[] =
   { 'v', M_COLLAPSED,		0,		NULL },
   { 'V', M_CRYPT_VERIFIED,	0,		NULL },
   { 'x', M_REFERENCE,		0,		eat_regexp },
+  { 'X', M_MIMEATTACH,		0,		eat_range },
   { 'y', M_XLABEL,		0,		eat_regexp },
   { 'z', M_SIZE,		0,		eat_range },
   { '=', M_DUPLICATED,		0,		NULL },
@@ -189,7 +190,7 @@ msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
 	  return (0);
 	}
 
-	fseek (msg->fp, h->offset, 0);
+	fseeko (msg->fp, h->offset, 0);
 	mutt_body_handler (h->content, &s);
       }
 
@@ -205,13 +206,13 @@ msg_search (CONTEXT *ctx, pattern_t* pat, int msgno)
       fp = msg->fp;
       if (pat->op != M_BODY)
       {
-	fseek (fp, h->offset, 0);
+	fseeko (fp, h->offset, 0);
 	lng = h->content->offset - h->offset;
       }
       if (pat->op != M_HEADER)
       {
 	if (pat->op == M_BODY)
-	  fseek (fp, h->content->offset, 0);
+	  fseeko (fp, h->content->offset, 0);
 	lng += h->content->length;
       }
     }
@@ -263,6 +264,11 @@ int eat_regexp (pattern_t *pat, BUFFER *s, BUFFER *err)
     snprintf (err->data, err->dsize, _("Error in expression: %s"), s->dptr);
     return (-1);
   }
+  if (!*buf.data)
+  {
+    snprintf (err->data, err->dsize, _("Empty expression"));
+    return (-1);
+  }
 
 #if 0
   /* If there are no RE metacharacters, use simple search anyway */
@@ -272,19 +278,24 @@ int eat_regexp (pattern_t *pat, BUFFER *s, BUFFER *err)
 
   if (pat->stringmatch)
   {
-    pat->str = safe_strdup (buf.data);
+    pat->p.str = safe_strdup (buf.data);
+    FREE (&buf.data);
+  }
+  else if (pat->groupmatch)
+  {
+    pat->p.g = mutt_pattern_group (buf.data);
     FREE (&buf.data);
   }
   else
   {
-    pat->rx = safe_malloc (sizeof (regex_t));
-    r = REGCOMP (pat->rx, buf.data, REG_NEWLINE | REG_NOSUB | mutt_which_case (buf.data));
+    pat->p.rx = safe_malloc (sizeof (regex_t));
+    r = REGCOMP (pat->p.rx, buf.data, REG_NEWLINE | REG_NOSUB | mutt_which_case (buf.data));
     FREE (&buf.data);
     if (r)
     {
-      regerror (r, pat->rx, err->data, err->dsize);
-      regfree (pat->rx);
-      FREE (&pat->rx);
+      regerror (r, pat->p.rx, err->data, err->dsize);
+      regfree (pat->p.rx);
+      FREE (&pat->p.rx);
       return (-1);
     }
   }
@@ -691,9 +702,11 @@ static int eat_date (pattern_t *pat, BUFFER *s, BUFFER *err)
 static int patmatch (const pattern_t* pat, const char* buf)
 {
   if (pat->stringmatch)
-    return !strstr (buf, pat->str);
+    return !strstr (buf, pat->p.str);
+  else if (pat->groupmatch)
+    return !mutt_group_match (pat->p.g, buf);
   else
-    return regexec (pat->rx, buf, 0, NULL, 0);
+    return regexec (pat->p.rx, buf, 0, NULL, 0);
 }
 
 static struct pattern_flags *lookup_tag (char tag)
@@ -733,12 +746,16 @@ void mutt_pattern_free (pattern_t **pat)
     tmp = *pat;
     *pat = (*pat)->next;
 
-    if (tmp->rx)
+    if (tmp->stringmatch)
+      FREE (&tmp->p.str);
+    else if (tmp->groupmatch)
+      tmp->p.g = NULL;
+    else if (tmp->p.rx)
     {
-      regfree (tmp->rx);
-      FREE (&tmp->rx);
+      regfree (tmp->p.rx);
+      FREE (&tmp->p.rx);
     }
-    FREE (&tmp->str);
+
     if (tmp->child)
       mutt_pattern_free (&tmp->child);
     FREE (&tmp);
@@ -802,6 +819,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	not = 0;
 	alladdr = 0;
 	break;
+      case '%':
       case '=':
       case '~':
 	if (implicit && or)
@@ -819,6 +837,7 @@ pattern_t *mutt_pattern_comp (/* const */ char *s, int flags, BUFFER *err)
 	tmp->not = not;
 	tmp->alladdr = alladdr;
         tmp->stringmatch = (*ps.dptr == '=') ? 1 : 0;
+        tmp->groupmatch  = (*ps.dptr == '%') ? 1 : 0;
 	not = 0;
 	alladdr = 0;
 
@@ -1045,7 +1064,7 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
     case M_WHOLE_MSG:
 #ifdef USE_IMAP
       /* IMAP search sets h->matched at search compile time */
-      if (Context->magic == M_IMAP && pat->stringmatch)
+      if (ctx->magic == M_IMAP && pat->stringmatch)
 	return (h->matched);
 #endif
       return (pat->not ^ msg_search (ctx, pat, h->msgno));
@@ -1111,6 +1130,12 @@ mutt_pattern_exec (struct pattern_t *pat, pattern_exec_flag flags, CONTEXT *ctx,
       return (pat->not ^ (h->env->spam && h->env->spam->data && patmatch (pat, h->env->spam->data) == 0));
     case M_DUPLICATED:
       return (pat->not ^ (h->thread && h->thread->duplicate_thread));
+    case M_MIMEATTACH:
+      {
+      int count = mutt_count_body_parts (ctx, h);
+      return (pat->not ^ (count >= pat->min && (pat->max == M_MAXRANGE ||
+                                                count <= pat->max)));
+      }
     case M_UNREFERENCED:
       return (pat->not ^ (h->thread && !h->thread->child));
   }
@@ -1142,7 +1167,7 @@ void mutt_check_simple (char *s, size_t len, const char *simple)
    * equivalences?
    */
   
-  if (!strchr (s, '~') && !strchr (s, '=')) /* yup, so spoof a real request */
+  if (!strchr (s, '~') && !strchr (s, '=') && !strchr (s, '%')) /* yup, so spoof a real request */
   {
     /* convert old tokens into the new format */
     if (ascii_strcasecmp ("all", s) == 0 ||
@@ -1180,15 +1205,8 @@ int mutt_pattern_func (int op, char *prompt)
   int i;
 
   strfcpy (buf, NONULL (Context->pattern), sizeof (buf));
-  if (mutt_get_field (prompt, buf, sizeof (buf), M_PATTERN | M_CLEAR) != 0)
+  if (mutt_get_field (prompt, buf, sizeof (buf), M_PATTERN | M_CLEAR) != 0 || !buf[0])
     return (-1);
-  if (!buf[0])
-  {
-    if (op == M_LIMIT)
-      strfcpy (buf, "~A", sizeof(buf));
-    else
-      return -1;
-  }
 
   mutt_message _("Compiling search pattern...");
   
