@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #if HAVE_CONFIG_H
@@ -24,7 +24,7 @@
 #include "mx.h"
 #include "url.h"
 #include "pop.h"
-#ifdef USE_SSL
+#if defined(USE_SSL)
 # include "mutt_ssl.h"
 #endif
 
@@ -274,7 +274,7 @@ int pop_open_connection (POP_DATA *pop_data)
     return -2;
   }
 
-#if defined(USE_SSL) || defined(USE_GNUTLS)
+#if defined(USE_SSL)
   /* Attempt STLS if available and desired. */
   if (!pop_data->conn->ssf && (pop_data->cmd_stls || option(OPTSSLFORCETLS)))
   {
@@ -301,11 +301,7 @@ int pop_open_connection (POP_DATA *pop_data)
 	mutt_error ("%s", pop_data->err_msg);
 	mutt_sleep (2);
       }
-#ifdef USE_SSL
       else if (mutt_ssl_starttls (pop_data->conn))
-#elif USE_GNUTLS
-      else if (mutt_gnutls_starttls (pop_data->conn))
-#endif
       {
 	mutt_error (_("Could not negotiate TLS connection"));
 	mutt_sleep (2);
@@ -428,7 +424,7 @@ int pop_query_d (POP_DATA *pop_data, char *buf, size_t buflen, char *msg)
     }
 #endif
 
-  mutt_socket_write_d (pop_data->conn, buf, dbg);
+  mutt_socket_write_d (pop_data->conn, buf, -1, dbg);
 
   c = strpbrk (buf, " \r\n");
   *c = '\0';
