@@ -13,7 +13,7 @@
  * 
  *     You should have received a copy of the GNU General Public License
  *     along with this program; if not, write to the Free Software
- *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *     Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */ 
 
 /* 
@@ -368,7 +368,7 @@ void rfc1524_free_entry(rfc1524_entry **entry)
   FREE (&p->editcommand);
   FREE (&p->printcommand);
   FREE (&p->nametemplate);
-  FREE (entry);
+  FREE (entry);		/* __FREE_CHECKED__ */
 }
 
 /*
